<?php

function mpg_get_datasets_lists()
{

    $datasets_storage = [];
    // This is .xlsx file that contain dataset name and url. But this URL provide data
    //	not in .xlsx but in .csv. So, get him and parse
    $source_file = 'https://docs.google.com/spreadsheets/d/1Bcow9bdsEXlv98mSIVTSP61SrkCB-y22QMD_GgatKzk/export?format=csv&id=1Bcow9bdsEXlv98mSIVTSP61SrkCB-y22QMD_GgatKzk&gid=0';

    try {
        $datasets_map = wp_remote_retrieve_body(wp_remote_get($source_file));

        if (!$datasets_map) {
            throw new Exception(__('Can not to get dataset list file. ', 'mpg'));
        }
        // datasets list in Google Drive File
        $datasets_array = csv_to_array($datasets_map);

        $datasets_in_use = get_option('multipages_datasets_in_use');
        $datasets_in_use = $datasets_in_use ? $datasets_in_use : [];


        foreach ($datasets_array as $dataset) {
            // Just way to skip "_comment" string
            if (strpos($dataset[0], '_') === 0) {
                continue;
            }

            if(in_array($dataset[1], $datasets_in_use)){
                $add_remove = '<a class="remove-from-library" href="' . $dataset[1] . '">Remove</a>';
            } else{
                $add_remove = '<a class="add-to-library" href="' . $dataset[1] . '">Add</a>';
            }

            $preview = '<a href="' . $dataset[1] . '" target="_blank">Preview</a>';

            //name, url, size, date
            $datasets_storage[] = [trim($dataset[0]), $dataset[2], $dataset[3], $dataset[4], $preview, $add_remove];
        }

        echo json_encode($datasets_storage);

    } catch (Exception $exception) {
        echo $exception->getMessage();
    }

    wp_die();
}

add_action('wp_ajax_get_datasets_list', 'mpg_get_datasets_lists');


function shutdown(){
    $isError = false;
    if ($error = error_get_last()){
        switch($error['type']){
            case E_ERROR:
            case E_CORE_ERROR:
            case E_COMPILE_ERROR:
            case E_USER_ERROR:
                $isError = true;
                break;
        }
    }

    if ($isError){
        echo json_encode([
            'success' => false,
            'message' => $error['message']
        ]);
        exit;
    }
}


add_action('wp_ajax_add_dataset_to_user_library', 'mpg_add_dataset_to_user_library');

function mpg_add_dataset_to_user_library(){

    try{


        register_shutdown_function('shutdown');



        $dataset_href = esc_sql($_POST['href']);
        $dataset_name = esc_sql($_POST['name']);

        $dataset_content = wp_remote_retrieve_body(wp_remote_get($dataset_href, array(
            'timeout' => 30,
            'sslverify' => false
        )));

        if (!$dataset_content) {
            throw new Exception(__('Can not to get dataset file. ', 'mpg'));
        }

        $file_resource = fopen(realpath(plugin_dir_path(__FILE__) . '../../') . '/datasets/' . $dataset_name . '.csv', 'w+');
        if (!$file_resource) {
            throw new Exception(__('Can not to open file for writing dataset library. Please, check directory permissions.', 'mpg'));
        }

        $write_handler = fwrite($file_resource, $dataset_content);

        if (!$write_handler) {
            throw new Exception(__('Can not write to dataset library file. Please, check directory permissions.', 'mpg'));
        }

        $datasets_in_use = get_option('multipages_datasets_in_use');

        if($datasets_in_use !== false){
            $datasets_in_use[] = $dataset_href;
            if(!update_option('multipages_datasets_in_use', $datasets_in_use)){
                throw new Exception(__('Can not update values in database', 'mpg'));
            }
        }else{
            if(!add_option('multipages_datasets_in_use', [$dataset_href])){
                throw new Exception(__('Can not to add new value to database', 'mpg'));
            }

        }

        echo json_encode(['success' => true]);

        wp_die();

    } catch(Exception $exception) {
        echo json_encode( [
            'success' => false,
            'message' => $exception->getMessage()
        ]);
        wp_die();
    }
}



add_action('wp_ajax_remove_dataset_from_user_library', 'mpg_remove_dataset_from_user_library');

function mpg_remove_dataset_from_user_library(){

    try{

        $dataset_href = esc_sql($_POST['href']);
        $dataset_name = esc_sql($_POST['name']);

        $datasets_in_use = get_option('multipages_datasets_in_use');

        if($datasets_in_use){

            if (($key = array_search($dataset_href, $datasets_in_use)) !== false) {
                unset($datasets_in_use[$key]);
            }

            if(!update_option('multipages_datasets_in_use', $datasets_in_use)){
                throw new Exception(__('Can not update values in database', 'mpg'));
            }

            $dataset_file_path = realpath(plugin_dir_path(__FILE__) . '../../') . '/datasets/' . $dataset_name . '.csv';

            if (!unlink($dataset_file_path)) {
                throw new Exception(__('Can not to delete dataset file from library. Please, check directory permissions.', 'mpg'));
            }
        }

        echo json_encode(['success' => true]);

        wp_die();

    } catch(Exception $exception) {
        echo json_encode( [
            'success' => false,
            'message' => $exception->getMessage()
        ]);

        wp_die();
    }
}

function mpg_datasets_library()
{

    echo '
        <div class="container-fluid">
            <div class="notification-container"></div>
            <h1 class="mt-4 mb-5">' . __('Datasets library', 'mpg') . '</h1>
            <table id="datasets-list" style="margin-top: 5rem" class="stripe table table-striped table-bordered">
                 <thead>
                    <tr>
                         <th>' . __('Name', 'mpg') . '</th>' .
                        '<th>' . __('Size (MB)', 'mpg') . '</th>' .
                        '<th>' . __('Last Updated', 'mpg') . '</th>' .
                        '<th>' . __('Provider', 'mpg') . '</th>' .
                        '<th>' . __('Download sample', 'mpg') . '</th>' .
                        '<th>' . __('Status', 'mpg') . '</th>' .
                    '</tr>
                </thead>
             </table>
             
             <p class="mt-5">' . __('Pay attention: large datasets may be not applicable, due to default memory limit.') . '</p>' .
                '<p>' . __('To increase memory limit, add "define(\'WP_MEMORY_LIMIT\', \'300M\');" to wp-config.php in root of WP installation folder.', 'mpg') . '</p>' .
         '</div>';

}

