<?php
/**
 * Created by PhpStorm.
 * User: Sanchoss
 * Date: 2019-04-15
 * Time: 17:02
 */

/**
 * Used for multistore.
 * Return name of site for appending to uploaded file name to avoid overriding.
 */

if (!defined('ABSPATH')) {
    exit;
}

require_once(realpath(dirname(__FILE__)) . "/../constants.php");

add_action('wp_ajax_mpg_get_tabs', 'mpg_get_tabs_callback');

/**
 * Return project list and some service info
 */
function mpg_get_tabs_callback()
{
    $projects_list = get_option('multipages_projects_list');

    $projects_list = $projects_list ? $projects_list : [];

    echo json_encode([
        'data' => $projects_list,
        'plugin_base_url' => MULTIPAGE_PLUGIN_URL,
        'is_pro_user' => mpg_app()->is_plan('Pro', true),
        'upgrade_url' => mpg_app()->get_upgrade_url()
    ]);

    wp_die();
}


add_action('wp_ajax_mpg_create_project', 'mpg_create_project_callback');

/**
 * Create project depend by sent from user data.
 */
function mpg_create_project_callback()
{
    $project_name = isset($_POST['project_name']) ? esc_sql($_POST['project_name']) : false;
    $id = isset($_POST['id']) ? esc_sql($_POST['id']) : false;

    if ($project_name and $id) {

        try {
            if (!update_option('multipages_project_' . $id,
                [
                    'name' => $project_name,
                    'project_id' => $id,
                    'template_id' => null,
                    'template_name' => null,
                    'scheduled_datetime' => null

                ])) {
                throw new Exception(__('Can not create new project', 'mpg'));
            }

            $current_projects_list = get_option('multipages_projects_list');

            if (is_string($current_projects_list)) {
                $current_projects_list = array();
            }

            $current_projects_list[] = [
                'name' => $project_name,
                'id' => $id
            ];


            if (!update_option('multipages_projects_list', $current_projects_list)) {
                throw new Exception(__('Can not update exist projects list'));
            }

            echo json_encode(['success' => true]);

        } catch (Exception $exception) {
            echo json_encode(['success' => false, 'notifications' => $exception->getMessage()]);
        }

    } else {
        _e('Project name or id is not specified', 'mpg');
    }

    wp_die();
}


add_action('wp_ajax_mpg_get_project_info', 'mpg_get_project_info_callback');

/**
 * Return project detailed info
 */
function mpg_get_project_info_callback()
{

    $project_id = isset($_POST['projectId']) ? esc_sql($_POST['projectId']) : false;

    if ($project_id) {
        $project_key = 'multipages_project_' . $project_id;
        $project_info = get_option($project_key);

        if ($project_info) {
            echo json_encode(['success' => true, 'data' => $project_info]);
        } else {
            echo json_encode([
                'success' => false,
                'notifications' => __('Project with specified project id was not found', 'mpg')
            ]);
        }
    } else {
        echo json_encode(['success' => true, 'data' => null]);
    }

    wp_die();
}


add_action('wp_ajax_mpg_set_page_template', 'mpg_set_page_template');

/**
 * Set up some page as a template for current project
 */
function mpg_set_page_template()
{
    try {
        if (isset($_POST['post_template_id']) and isset($_POST['project_id']) and isset($_POST['post_template_name'])) {
            $project_id = (int)$_POST['project_id'];

            $project_key = 'multipages_project_' . $project_id;
            if (update_value_in_db(
                $project_key,
                ['template_id', 'template_name'],
                [
                    (int)$_POST['post_template_id'],
                    esc_sql($_POST['post_template_name'])
                ]
            )) {

                echo json_encode(['success' => true]);

            } else {
                throw new Exception(__('Project with specified id was not found', 'mpg'));
            }
        }

    } catch (Exception $exception) {
        echo json_encode(['success' => false, 'notifications' => $exception->getMessage()]);
    }

    wp_die();
}


add_action('wp_ajax_mpg_rename_project', 'mpg_rename_project');

/**
 * Rename project
 */
function mpg_rename_project()
{

    $project_id = isset($_POST['project_id']) ? esc_sql($_POST['project_id']) : false;
    $new_name = isset($_POST['project_name']) ? esc_sql($_POST['project_name']) : false;

    if ($project_id and $new_name) {

        $current_projects_list = get_option('multipages_projects_list');

        $list_storage = [];
        foreach ($current_projects_list as $key => $project) {

            if ((int)$project['id'] === (int)$project_id) {
                $project['name'] = $new_name;
                $list_storage[$key] = $project;
            } else {
                $list_storage[] = $project;
            }
        }

        // Update name in project list
        update_option('multipages_projects_list', $list_storage);

        $project_key = 'multipages_project_' . $project_id;
        $project_object = get_option($project_key);

        if ($project_object) {
            $project_object['name'] = $new_name;
            // Update name in project object
            update_option($project_key, $project_object);
        }

        echo json_encode(['success' => true]);
    }

    wp_die();
}


add_action('wp_ajax_mpg_delete_project', 'mpg_delete_project');

/**
 * Delete project with data file and meta info
 */
function mpg_delete_project()
{

    $project_id = isset($_POST['project_id']) ? (int)$_POST['project_id'] : false;

    if ($project_id) {

        $current_projects_list = get_option('multipages_projects_list');

        foreach ($current_projects_list as $key => $project) {
            if ((int)$project['id'] === $project_id) {
                // Delete array element (project) if id is equal
                unset($current_projects_list[$key]);
                break;
            }
        }

        $data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';

        try {
            if (update_option('multipages_projects_list', $current_projects_list) and
                delete_option('multipages_project_' . $project_id)) {

                if (file_exists($data_file_path)) {
                    unlink($data_file_path);
                }

                echo json_encode(['success' => true]);

            } else {

                throw new Exception('Error occurred while deleting project');
            }

        } catch (Exception $e) {
            echo json_encode(['success' => false, 'notifications' => $e->getMessage()]);
        }
    }
    wp_die();
}


add_action('wp_ajax_mpg_get_data_file_content', 'mpg_get_user_data_and_search_handler');

/**
 * Get user data file body and some service data
 * and this method caled when user type some keyword to search!
 */
function mpg_get_user_data_and_search_handler()
{
    $project_id = isset($_POST['projectId']) ? esc_sql($_POST['projectId']) : null;
    $search_input = isset($_POST['searchInput']) ? esc_sql($_POST['searchInput']) : null;
    $page = isset($_POST['page']) ? (int)($_POST['page']) - 1 : 0;

    if (!$project_id) {
        echo json_encode(['success' => false, 'error' => __('Project id is not specified')]);
    }

    $search_result_storage = [];
    $data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';

    $data_file_content = file_exists($data_file_path) ? file_get_contents($data_file_path) : false;
    $project_document = get_option('multipages_project_' . $project_id);

    if ($data_file_content and $project_document) {

        $data_file_content_array = json_decode($data_file_content, true);

        if ($search_input) {

            $search_string = trim(strtolower($search_input));

            foreach ($data_file_content_array as $row) {

                if (array_search($search_string, array_map('strtolower', $row), true) !== false) {
                    $search_result_storage[] = $row;
                }
            }

            if (!count($search_result_storage)) {
                echo json_encode(['success' => true, 'data' => null]);
                wp_die();
            }

        } else {
            // Cut shortcodes from begining of file
            array_shift($data_file_content_array);
        }


        $nb_elem_per_page = 30;

        $all_pages_count = $search_input ? count($search_result_storage) / $nb_elem_per_page :
            count($data_file_content_array) / $nb_elem_per_page;

        // $all_pages_count may contain number like 2.43. It's mean, that full pages is 3.
        // But if number is round (like a 5), so, just return this number, without +1 to pages count.
        $all_pages_count = is_int($all_pages_count) ? $all_pages_count : (int)$all_pages_count + 1;

        // Limit maximum page, for cases when user request 6-th page, when all count is 5.
        if ($page > $all_pages_count) {
            $page = $all_pages_count;
        }

        $data_file_results = array_slice($data_file_content_array, $page * $nb_elem_per_page, $nb_elem_per_page);
        $search_result_storage = array_slice($search_result_storage, $page * $nb_elem_per_page, $nb_elem_per_page);

        $data_table = array_merge(
            ['success' => true],
            ['shortcodes' => json_decode($project_document['csv_settings_shortcodes'], true)],
            ['urls' => json_decode($project_document['csv_settings_urls'], true)],
            ['data' => $search_input ? $search_result_storage : $data_file_results],
            ['pages' => $all_pages_count],
            ['cron_execution_datetime' => $project_document['scheduled_datetime']]
        );

        echo json_encode($data_table);
        wp_die();

    } else {
        // Need to import demo. Lets show this block on front
        echo json_encode([
            'success' => true,
            'data' => null,
            'cron_execution_datetime' => $project_document['scheduled_datetime']
        ]);
    }

    wp_die();
}


add_action('wp_ajax_mpg_import_demo_data', 'mpg_import_demo_data_callback');

/**
 * Make import demo data for specified project
 */
function mpg_import_demo_data_callback()
{

    $project_id = (int)$_POST['project_id'];

    $default_data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/default-settings.json';
    $data_json = file_get_contents($default_data_file_path);

    $user_data_processing_result = user_data_processing(
        json_decode($data_json, true),
        $project_id);

    if ($user_data_processing_result) {
        echo json_encode(['success' => true]);
    } else {
        echo json_encode(['success' => false, 'notifications' => $user_data_processing_result->getMessage()]);
    }

    wp_die();
}

add_action('wp_ajax_mpg_processing_user_file_upload', 'mpg_processing_user_file_upload_callback');

function mpg_processing_user_file_upload_callback()
{

    if (isset($_FILES['multipages_csv_settings']) and isset($_REQUEST['project_id'])) {

        $csv = file_get_contents($_FILES['multipages_csv_settings']['tmp_name']);

        $project_id = (int)$_REQUEST['project_id'];
        
        $response = ['success' => null, 'notifications' => []];

        // todo: додати перевірку що цей файл - .csv по [tmp_ext] ?
        $data_json = csv_to_array($csv);

        if (!mpg_app()->is_plan('Pro', true)) {

            if (count($data_json) > 51) {
                $response['success'] = true;
                $response['notifications'][] = [
                    type => 'warning',
                    text => __('Your CSV file has more than 50 rows. This is not available in the current version, upgrade to PRO version to have unlimited pages. Please, upgrade to PRO version.', 'mpg')
                ];
            }
        }

        try {

            $shortcodes_array = generate_shortcodes_array($data_json);

            if ($shortcodes_array['url_column_index'] === false) {
                // It's mean, that users .csv haven't URL column.

                echo json_encode([
                    'success' => true,
                    'columns' => implode(',', $shortcodes_array['shortcodes_array'])
                ]);
                wp_die();

            } else {

                echo user_data_processing($data_json, $project_id);
                wp_die();
            }

            $response['success'] = true;
            $response['notifications'][] = [
                type => 'success',
                text => __('Settings was successfully updated', 'mpg')
            ];

        } catch (Exception $data_processing_exception) {
            $response['success'] = false;
            $response['notifications'][] = [
                type => 'danger',
                text => $data_processing_exception->getMessage()
            ];
        }

        echo json_encode($response);
        wp_die();
    }
}


function getSiteName()
{
    global $blog_id;

    $siteName = false;

    if (is_multisite()) {
        $current_blog_details = get_blog_details(array('blog_id' => $blog_id));
        $siteName = $current_blog_details->path === '/' ? 'main' : str_replace('/', '', $current_blog_details->path);
    } else {
        $siteName = str_replace(getDomain(), '', trim(home_url('/', 'relative'), '/'));
    }

    if ($siteName === false or empty(trim($siteName))) {
        return 'single';
    }

    return str_replace('/', '-', trim($siteName));
}

function getSiteUrl()
{

    global $blog_id;

    if (is_multisite()) {
        $current_blog_details = get_blog_details(array('blog_id' => $blog_id));
        $siteName = $current_blog_details->path === '/' ? 'main' : str_replace('/', '', $current_blog_details->path);
    } else {
        $siteName = str_replace(getDomain(), '', trim(home_url('/', 'relative'), '/'));
    }

    return trim($siteName);
}

// Return site URL
function getDomain()
{
    return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]";
}


// Return the path of URL
function getRequestUri()
{
    global $wp;
    $fullUrlPath = home_url($wp->request);
    $siteUrl = get_site_url();
    $currentURL = urldecode(str_ireplace($siteUrl, "", $fullUrlPath) . "/");

    return $currentURL;
}


/**
 * replace shortcodes in head section if exist
 *
 * @param $html
 *
 * @return string
 */
function multipage_replace_data($html)
{

    $metadata_array = get_urls_and_project_id();

    $templatePost = get_post($metadata_array['template_id']);
    $currentPost = get_post();

    if ($templatePost->ID == $currentPost->ID) {

        $redirect_rules = get_redirect_rules();
        // If requested URL is available in array of all URL's
        $index_of_element = array_search(getRequestUri(), array_column($redirect_rules, 'url'));


        $urls_array = array_column($redirect_rules, 'url');
        $project_id = $redirect_rules[$index_of_element]['project_id'];


        return mpgScReplacer($html, [], $urls_array, $project_id);
    }
}

/**
 * Set correct encoding. Create preg template.
 * Replace short codes from WP content to data from short code data table.
 *
 * @param $content string - content string from Word Press
 * @param array $dataSet
 * @param $redirect_rules
 * @param $project_id
 *
 * @return string
 */
function mpgScReplacer($content, $dataSet = [], $redirect_rules, $project_id)
{
    $currentPost = get_post();
    $currentPermalink = get_permalink($currentPost);

    $project_object = get_option('multipages_project_' . $project_id);
    $shortCodes = $project_object ? json_decode($project_object['csv_settings_shortcodes'], true) : false;

    $rightURL = false;
    $UrlPath = getRequestUri();

    $data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';
    $default_data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/default-settings.json';

    $data_file_content = file_exists($data_file_path) ? file_get_contents($data_file_path) : file_get_contents($default_data_file_path);
    $data_file_content_array = json_decode($data_file_content, true);

    $siteUrl = get_site_url();

    if (in_array($UrlPath, $redirect_rules)) {
        $rightURL = get_option('siteurl') . $UrlPath;
    }

    // do action with short codes
    $urlColumn = false;
    foreach ($shortCodes as $key => &$shortCode) {

        if ($shortCode == 'mpg_url') {
            $urlColumn = $key;
        } // find the URL column

        $shortCode = "/{{" . str_replace('/', '\/', $shortCode) . "}}/"; // create template for preg_replace function

    }

    // this works for mpgGroupAction
    if (!empty($dataSet)) {
        $dataString = '';
        foreach ($dataSet as $dsValue) {
            // // get data set by each
            if ($urlColumn !== false) {
                $dsValue[$urlColumn] = $siteUrl . $dsValue[$urlColumn];
            } // add to URL column correct site URL
            $dataString .= preg_replace($shortCodes, $dsValue, $content);
        }
        return $dataString;
    }


    // replace the template permalink to urls in settings table
    if ($rightURL) {
        $content = str_replace($currentPermalink, $rightURL, $content);
        $content = str_replace(urlencode($currentPermalink), urlencode($rightURL), $content);
    }

    $strings = $data_file_content_array[$UrlPath];

    if ($urlColumn !== false) {
        $strings[$urlColumn] = $siteUrl . $strings[$urlColumn]; // add to URL column correct site URL
    }

    return (preg_replace($shortCodes, $strings, $content));
}

/**
 * @param $type
 * @param $text
 * @param bool $dismissible
 */
function mpg_show_notification($type, $text, $dismissible = false)
{

    $notification_block = '<div class="alert alert-' . $type . ' alert-dismissible fade show" role="alert">'
        . $text;
    if ($dismissible) {
        $notification_block .= '<button type="button" class="close" data-dismiss="alert" aria-label="Close">
                <span aria-hidden="true">&times;</span>
            </button>';
    }

    echo $notification_block . '</div>';
}


/**
 * @param $value
 * @param int $options
 * @param int $depth
 * @param bool $utfErrorFlag
 *
 * @return false|string
 * @throws Exception
 */
function safe_json_encode($value, $options = 0, $depth = 512, $utfErrorFlag = false)
{
    $encoded = json_encode($value, $options, $depth);
    switch (json_last_error()) {
        case JSON_ERROR_NONE:
            return $encoded;
        case JSON_ERROR_DEPTH:
            throw new Exception(__('Maximum stack depth exceeded', 'mpg'));
        case JSON_ERROR_STATE_MISMATCH:
            throw new Exception(__('Underflow or the modes mismatch', 'mpg'));
        case JSON_ERROR_CTRL_CHAR:
            throw new Exception(__('Unexpected control character found', 'mpg'));
        case JSON_ERROR_SYNTAX:
            throw new Exception(__('Syntax error, malformed JSON', 'mpg'));
        case JSON_ERROR_UTF8:
            $clean = utf8ize($value);
            if ($utfErrorFlag) {
                throw new Exception(__('UTF8 encoding error', 'mpg'));
            }

            return safe_json_encode($clean, $options, $depth, true);
        default:
            throw new Exception(__('Unknown error', 'mpg'));
    }
}

/**
 * @param $mixed
 *
 * @return array|string
 */
function utf8ize($mixed)
{
    if (is_array($mixed)) {
        foreach ($mixed as $key => $value) {
            $mixed[$key] = utf8ize($value);
        }
    } else if (is_string($mixed)) {
        return utf8_encode($mixed);
    }

    return $mixed;
}

function get_split_char($str) {
    $replaced_string = preg_replace('/".+"/isU', '*', $str);
    $possible_separators = ["\t", "\n", "\r"];
    $result = '';
    $i = -1;
    foreach($possible_separators as $separator) {
        if(($n = sizeof(explode($separator, $replaced_string))) > $i) {
            $i = $n;
            $result = $separator;
        }
    }
    return $result;
}

function remove_utf8_bom($text)
{
    $bom = pack('H*','EFBBBF');
    $text = preg_replace("/^$bom/", '', $text);
    return $text;
}

/**
 * @param $csv_string
 *
 * @return array
 */
function csv_to_array($csv_string)
{

    $csv_encoding = mb_detect_encoding($csv_string);

    if($csv_encoding !== 'UTF-8'){
        $csv_string = iconv($csv_encoding, 'UTF-8', $csv_string);
    }

    $csv_string = remove_utf8_bom($csv_string);
    
    $line_break_symbol = "\n";//get_split_char($csv_string);

    return array_map("str_getcsv", explode($line_break_symbol, $csv_string));
}

function mpg_csv_validator($csv_string){

    $validation_results = [];
 
    // 1. Проверяем. яввляется ли кодировка utf-8
    $csv_encoding = mb_detect_encoding($csv_string);

    if($csv_encoding !== 'UTF-8'){
        array_push($validation_results, __('Pay attention: file may be shown wrong, because current encoding is', 'mpg') . ' ' . $csv_encoding . ', ' . __('but needed is UTF-8.', 'mpg'));
    }

    // 2. Проверяем корретность разделителя значений

    if(count(str_getcsv($csv_string, ';', "\n")) > 1){
        array_push($validation_results, __('Pay attention: file may be shown wrong, due to inappropriate values separator. Probably, semicolon used, but comma is needed.', 'mpg'));
    } 

    // echo $validation_results;

    return $validation_results;
}


/**
 * @param $raw_link
 *
 * @return string
 * @throws Exception
 */
function get_direct_csv_link($raw_link)
{
    // false = substring was not found in target string
    if (strpos($raw_link, 'docs.google.com') !== false or strpos($raw_link, 'drive.google.com') !== false) {

        $documentId = str_replace([
            'https://docs.google.com/spreadsheets/d/',
            'https://drive.google.com/file/d/',
            '/view?usp=sharing',
            '/edit?usp=sharing'
        ], ['', '', '', ''], $raw_link);

        return 'https://docs.google.com/spreadsheets/d/' . $documentId . '/export?format=csv&id=' . $documentId;
    }

    // If link is not Google Docs or Drives - let's think that it's direct csv link.
    return $raw_link;
}


/**
 * Main function that fire when time is came.
 *
 * @param $project_id
 * @param $raw_link
 * @param $cron_notification
 * @param $periodicity
 * @param $notification_email
 *
 * @throws Exception
 */
function scheduled_cron_handler($project_id, $raw_link, $cron_notification, $periodicity, $notification_email)
{
    try {

        $csv_link = get_direct_csv_link($raw_link);

        $users_csv_string = file_get_contents($csv_link);

        if (!$users_csv_string) {
            throw new Exception(__('Looks like your file is not valid.', 'mpg'));
        }

        $scheduled_user_data_array = csv_to_array($users_csv_string);

        $shortcodes_array = generate_shortcodes_array($scheduled_user_data_array);

        $urls_and_strings_array = generate_urls_and_strings_array($scheduled_user_data_array, $shortcodes_array);

        if ($shortcodes_array['url_column_index'] !== false) {

            // If user upload file with non utf-8 symbols, generic json_encode not working.
            // So, I used wrapper for that, that convert all strings to utf-8

            update_value_in_db(
                'multipages_project_' . $project_id,
                ['csv_settings_urls', 'csv_settings_shortcodes'],
                [
                    safe_json_encode($urls_and_strings_array['urls_array']),
                    safe_json_encode($shortcodes_array['shortcodes_array'])
                ]
            );

            $data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';

            // set dynamic link with project id
            $user_data_file_resource = fopen($data_file_path, 'w+');

            if (!$user_data_file_resource) {
                throw new Exception(__('Error opening template file.', 'mpg'));
            }

            $data_without_headers = json_encode($urls_and_strings_array['strings_array']);

            if (!$data_without_headers) {
                throw new Exception(__('Your .csv file is not valid, corrupted, or has syntax errors', 'mpg'));
            }

            if (!fwrite($user_data_file_resource, $data_without_headers)) {
                throw new Exception(__('Can not write your data to file. Please, check permission'), 'mpg');
            }

            if ($cron_notification === 'every-time') {
                mail($notification_email, __('MPG schedule execution report: ok', 'mpg'),
                    'Hi. <br>Scheduled task was completed successfully.');
            }

        } else {
            throw new Exception('In your table missing "url" column');
        }


    } catch (Exception $e) {
        if ($cron_notification === 'errors-only') {
            fwrite(fopen('log.txt', 'w+'), $e->getMessage());
            mail($notification_email, 'MPG schedule execution report: failed',
                'Hi. <br>In process of execution the next error occurred: ' . $e->getMessage()
            );
        }
    }

    // If cron task is repetitive - we should't delete this option. Because task still isn't completed
    if ($periodicity === 'once') {

        update_value_in_db('multipages_project_' . $project_id,
            ['scheduled_datetime', 'cron_arguments'], [null, null]);
    }
}

/**
 * Additional intervals for cron,
 * because WP hasn't weekly and monthly periods
 *
 * @param $schedules
 *
 * @return mixed
 */
function cron_monthly($schedules)
{
    $schedules['monthly'] = array(
        'interval' => 60 * 60 * 24 * 30,
        'display' => __('Monthly', 'mpg')
    );

    return $schedules;
}

/**
 * Additional intervals for cron,
 * because WP hasn't weekly and monthly periods
 *
 * @param $schedules
 *
 * @return mixed
 */
function cron_weekly($schedules)
{
    $schedules['weekly'] = array(
        'interval' => 60 * 60 * 24 * 7,
        'display' => __('Weekly', 'mpg')
    );

    return $schedules;
}


/**
 * Function that create default tab in case,
 * if user has install and activate MPG at first time
 */
function mpg_create_default_tab()
{

    $project_name = __('Default tab', 'mpg');
    $id = 1;  // Default tab always have id 1;
    $page_title = '{{mpg_city}}, {{mpg_state}} page';

    try {
        if (!update_option('multipages_project_' . $id,
            [
                'name' => $project_name,
                'project_id' => $id,
                'template_id' => create_demo_page($page_title),
                'template_name' => $page_title,
                'scheduled_datetime' => null
            ])) {
            throw new Exception(__('Can not create new project', 'mpg'));
        }


        if (!update_option('multipages_projects_list', array(['name' => $project_name, 'id' => $id]))) {
            throw new Exception(__('Can not update exist projects list'));
        }

    } catch (Exception $e) {
        echo $e->getMessage();
    }
}


/**
 * Helper function that create demo page,
 * or return demo page id if any exist
 *
 * @param $page_title
 *
 * @return |null
 */
function create_demo_page($page_title)
{
    $exist_page = get_page_by_title($page_title);
    $page_id = null;

    // If demo page is not created yet, or this page in trash, let's create new one.
    if (!$exist_page or ($exist_page and $exist_page->post_status === 'trash')) {

        $page_id = wp_insert_post([
            'post_title' => $page_title,
            'post_content' =>
                '<a href="{{mpg_url}}">{{mpg_city}}, {{mpg_state}}</a>
	                 <p>Did you know that {{mpg_city}} has a population of {{mpg_population}} people? And yes, this is <u>generated content</u>!</p>
	                 <p>This is an example of a customized page that MPG has generated for {{mpg_city}}, {{mpg_state}}. MPG Pro can generate an unlimited number of pages and substitute shortcodes from your CSV file to create unique content for each one. Using MPG\'s filters and features you can make thousands of relevant landing pages, driving converting traffic and growing your business.</p>
	                 <p>Check out this example of an MPG filter that returns all rows containing a giving query and can be served dynamically. View the code of this page to learn how it\'s used. </p>
	                 <p>Here are other pages from {{mpg_state}} that MPG has generated:</p>
	                 <ul>[mpg where="mpg_state={{mpg_state}}"]<li><a href="{{url}}">{{city}}, {{state}}</a></li>[/mpg]</ul>
	                 <p> Learn more about how MPG works, tutorials and how to get the plugin at:  <br><a href="https://www.mpgwp.com/">www.mpgwp.com</a></p>',
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => 'page'
        ]);

        return $page_id;
    } else {
        return $exist_page->ID;
    }
}


function user_data_processing($data_json, $project_id)
{

    $sitemap_name = isset($_POST['sitemap_file_name']) ? esc_sql($_POST['sitemap_file_name']) : null;
    $sitemap_max_url = isset($_POST['sitemap_max_url']) ? esc_sql($_POST['sitemap_max_url']) : null;
    $sitemap_update_freq = isset($_POST['sitemap_frequency']) ? esc_sql($_POST['sitemap_frequency']) : null;
    $add_to_robots = isset($_POST['sitemap_robot']) ? esc_sql($_POST['sitemap_robot']) : null;

    // If user purchased on a FREE plan, trim all his incoming
    // datasets at 50 rows (51 because first is a header)
    if (!mpg_app()->is_plan('Pro', true)) {
        $data_json = array_slice($data_json, 0, 51);
    }

    $shortcodes_array = generate_shortcodes_array($data_json);

    $urls_and_strings_array = generate_urls_and_strings_array($data_json, $shortcodes_array);

    $urls_array = $urls_and_strings_array['urls_array'];
    array_shift($urls_array);

    $sitemap_result = [];

    try {
        if (isset($_POST['create_sitemap']) and class_exists('MPG_SitemapGenerator')) {

            if (mpg_sitemap_generator($urls_array, $sitemap_name, $sitemap_max_url, $sitemap_update_freq, $add_to_robots)) {

                $sitemap_filename = $sitemap_name ? $sitemap_name . '.xml' : 'multipage-sitemap.xml';
                $sitemap_result[] = ['type' => 'success', 'text' => __('Sitemap was created. ', 'mpg') . getDomain() . '/' . getSiteUrl() . '/' . $sitemap_filename];

            } else {
                throw new Exception(__('Can not to create sitemap. Unknown error occurred.'));
            }
        }

        update_value_in_db(
            'multipages_project_' . $project_id,
            ['csv_settings_urls', 'csv_settings_shortcodes'],
            [json_encode($urls_array), json_encode($shortcodes_array['shortcodes_array'])]
        );

        $data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';

        $fileResource = fopen($data_file_path, "w+");
        if (!$fileResource) {
            throw new Exception(__('Error opening template file.', 'mpg'));
        }

        $template_data = json_encode($urls_and_strings_array['strings_array']);

        if (!$template_data) {
            throw new Exception(__('Your .csv file is not valid, corrupted, or has syntax errors', 'mpg'));
        }

        if (!fwrite($fileResource, $template_data)) {
            throw new Exception(__('Can not write your data to file. Please, check permission'), 'mpg');
        }

        $sitemap_result[] = ['type' => 'success', 'text' => __('Data  file was successfully applied.', 'mpg')];


        return json_encode(['success' => true, 'notifications' => $sitemap_result, 'dismissible' => true]);


    } catch (Exception $exception) {
        return json_encode(['success' => false, 'notifications' => [$exception->getMessage()]]);
    }
}


/**
 * Set option in wp_option, that redirect is needed.
 */
function mpg_set_activation_option()
{
    add_option('multipages_is_first_launch', true);
}

/**
 * Fire when plugin is activated.
 */
function mpg_activation_tasks()
{

    if (get_option('multipages_is_first_launch', false)) {

        delete_option('multipages_is_first_launch');

        if (!isset($_GET['activate-multi'])) {

            $projects_list = get_option('multipages_projects_list');

            // If there are is not at least one tab
            if (count($projects_list) === 0) {
                // Redirect user to "Welcome screen" after plugin activation.
                wp_redirect(admin_url('admin.php?page=multi-pages-wizard'));

            } else {
                // to dashboard
                wp_redirect(admin_url('admin.php?page=multi-pages-plugin'));
            }
        }
    }
}

/**
 * Remove cron task when user deactivate plugin
 */
function mpg_set_deactivation_option()
{
    wp_clear_scheduled_hook('schedule_execution');

    delete_option('multipages_is_first_launch');

}


/**
 * Create sitemap file depend on user .csv file data.
 *
 * @param $urlsArray
 * @param $sitemap_name
 * @param $sitemap_max_url
 * @param $sitemap_update_freq
 * @param $add_to_robots
 *
 * @return bool
 */
function mpg_sitemap_generator($urlsArray, $sitemap_name, $sitemap_max_url, $sitemap_update_freq, $add_to_robots)
{
    try {
        // Like /Lib/Documents/wordpress/
        $siteRootPath = ABSPATH;

        // Like a localhost
        $baseUrl = getSiteUrl();

        $frenquency = $sitemap_update_freq ? $sitemap_update_freq : 'monthly';


        $siteRootPathWithoutSlash = substr($baseUrl, -1) === '/' ? getDomain() . $baseUrl : getDomain() . $baseUrl . '/';

        $siteMap = new MPG_SitemapGenerator($siteRootPathWithoutSlash, $siteRootPath);


        $siteMap->addUrl($siteRootPath, date('Y-m-d'), $frenquency, '1');

        foreach ($urlsArray as $url) {

            $siteMap->addUrl(getDomain() . str_replace('//', '/', '/' . getSiteUrl() . $url), null, $frenquency, '1');
        }

        if ($sitemap_name) {
            $siteMap->setSitemapFileName($sitemap_name);
        }
        if ($sitemap_max_url) {
            $siteMap->setMaxURLsPerSitemap($sitemap_max_url);
        } // set max URLs

        $siteMap->createSitemap(); // create sitemap
        $siteMap->writeSitemap(); // write sitemap as file

//		$sitemap_filename = $sitemap_name ? $sitemap_name . '.xml' : 'multipage-sitemap.xml';

        if ($add_to_robots) {
            $siteMap->updateRobots();
        } // update robots.txt file

        return true;

    } catch (Exception $exception) {

        return false;

    }
}


/**
 * Helper function for bulk update values in DB.
 *
 * @param $key
 * @param $objects_properties_names
 * @param $values
 *
 * @return bool
 * @throws Exception
 */
function update_value_in_db($key, $objects_properties_names, $values)
{
    $current_object = get_option($key);

    if (!$current_object) {
        throw new Exception(__('Can not get object. Maybe project id is wrong', 'mpg'));
    }

    if (count($objects_properties_names) !== count($values)) {
        throw new Exception(__('Incoming arrays has not match elements count', 'mpg'));
    }

    foreach ($objects_properties_names as $index_num => $prop_name) {

        $current_object[$prop_name] = $values[$index_num];

    }

    return !!update_option($key, $current_object);

}

/**
 *
 * This function needed for cut heading (titles of columns) of user csv.
 * Return headings and numeric index of "url" columns
 *
 * @param $data_json
 *
 * @return array | bool
 */
function generate_shortcodes_array($data_json)
{
    $urlColumn = false;
    $shortcodes_array = [];

    foreach (array_shift($data_json) as $key => $shortcode_name) {
        $s_name = str_replace(' ', '_', $shortcode_name);
        $s_name = strtolower($s_name);
        $shortcodes_array[] = 'mpg_' . $s_name;
        if ($shortcode_name == 'url') {
            $urlColumn = $key;
        }
    }

    // if URL column wasn't found, url_column _index will contain false;

    return ['shortcodes_array' => $shortcodes_array, 'url_column_index' => $urlColumn];
}


/**
 * This function needed for cut body of users .csv without headings (titles of columns);
 * Return array with "url" of rows, and users "strings" array
 *
 * @param $data_json
 * @param $shortcodes_array
 *
 * @return array
 */

function generate_urls_and_strings_array($data_json, $shortcodes_array)
{
    $urlsArray = [];
    $stringsArray = [];
    foreach ($data_json as $key => $value) {

        // if URL value is empty or null then go to next iteration
        if (empty($value[$shortcodes_array['url_column_index']])) {
            continue;
        }
        $urlPath = trim($value[$shortcodes_array['url_column_index']]);
        $urlsArray[$urlPath] = $urlPath;
        $stringsArray[$urlPath] = $value;
    }

    return ['urls_array' => $urlsArray, 'strings_array' => $stringsArray];

}

function get_redirect_rules()
{
    global $wpdb;

    // array of multi URLs
    $redirect_rules = [];

    $all_projects_data = $wpdb->get_results("SELECT option_value FROM " . $wpdb->prefix . "options WHERE option_name LIKE 'multipages_project_%'");

    foreach ($all_projects_data as $project_object) {

        $project = unserialize($project_object->option_value);

        if (isset($project['csv_settings_urls'])) {

            $urls_array = json_decode($project['csv_settings_urls'], true);

            foreach ($urls_array as $single_url) {
                // Make arrays with urls from different projects together. It's like flatten array.
                $redirect_rules[] = [
                    'url' => $single_url,
                    'template_id' => $project['template_id'],
                    'project_id' => $project['project_id']
                ];
            }
        }
    }

    return $redirect_rules;
}

add_action('wp_ajax_mpg_change_permalink_structure', 'mpg_change_permalink_structure');

function mpg_change_permalink_structure()
{

    if (update_option('permalink_structure', '/%postname%/')) {
        echo json_encode(['success' => true, 'notifications' => __('Permalink structure was changed', 'mpg')]);
    } else {
        echo json_encode(['success' => false, 'notifications' => __('Permalink structure was not changed', 'mpg')]);
    }

    wp_die();
}

function getAllTepmlatesIds()
{
    global $wpdb;

    $cache = wp_cache_get('getAllTepmlatesIds');
    if ($cache) {
        return $cache;
    }

    $templates_ids = [];

    $all_projects_data = $wpdb->get_results("SELECT option_value FROM " . $wpdb->prefix . "options WHERE option_name LIKE 'multipages_project_%'");

    if ($all_projects_data) {
        foreach ($all_projects_data as $project_object) {
            $project = unserialize($project_object->option_value);

            if (isset($project['template_id'])) {
                $templates_ids[] = $project['template_id'];
            }
        }
    }

    wp_cache_add('getAllTepmlatesIds', $templates_ids);
    return $templates_ids;
}