<?php
/**
 * User: Daniil Krok
 * Date: 10.10.2018
 */


if (!defined('ABSPATH')) exit;

add_action('admin_menu', 'mpg_plugin_setup_menu', 9, 0);

function mpg_plugin_setup_menu()
{
    add_action('admin_init', 'mpg_register_mysettings');

    add_menu_page(__('Multiple Pages Generator by Porthas', 'mpg'), __('Multiple Pages Generator', 'mpg'), 'edit_pages', 'multi-pages-plugin', 'mpg_settings_page', MULTIPAGE_PLUGIN_URL . 'img/logo_mpg.svg');
    add_submenu_page('multi-pages-plugin', __('Dashboard', 'mpg'), __('Dashboard', 'mpg'), 'edit_pages', 'multi-pages-plugin', 'mpg_settings_page');
    // First parameter is null - it's way to hide page from menu
    add_submenu_page(null, __('Welcome screen', 'mpg'), __('Welcome screen', 'mpg'), 'edit_pages', 'multi-pages-welcome-screen', 'mpg_welcome_screen');
    add_submenu_page(null,__('Wizard', 'mpg'), __('Wizard', 'mpg'), 'edit_pages', 'multi-pages-wizard', 'mpg_wizard');
    add_submenu_page(null,__('Dashboard', 'mpg'), __('Dashboard', 'mpg'), 'edit_pages', 'multi-pages-dashboard-limited', 'mpg_dashboard_limited');

    add_submenu_page('multi-pages-plugin',__('Datasets Library', 'mpg'), __('Datasets Library', 'mpg'), 'edit_pages', 'multi-pages-datasets-library', 'mpg_datasets_library');
}

function mpg_register_mysettings()
{
    register_setting('multipages-settings-group', 'mpg_csv_settings');
}

add_action('init', 'mpg_loader');

function mpg_loader()
{
    require_once(__DIR__ . '/csv/autoload.php');
    require_once(__DIR__ . '/MPG_SitemapGenerator.php');
    require_once(__DIR__ . '/assets.php');
    require_once(__DIR__ . '/welcome_screen.php');
    require_once(__DIR__ . '/functions.php');
    require_once(__DIR__ . '/wizard/wizard_processing.php');
    require_once(__DIR__ . '/dashboard_limited.php');
}

function mpg_settings_page()
{
	// If user visit Dashboard first time, need to redirect his to Dashboard limited (to run Wizard)
    $project_list = get_option('multipages_projects_list');
    if(!$project_list or count($project_list) === 0){
        echo '<script>location.href="admin.php?page=multi-pages-dashboard-limited"</script>';
    } ?>

    <div class="container-fluid mpg-dashboard">

        <div class="notification-container"></div>

        <?php

        require_once(__DIR__ . '/schedule_csv_processing.php');

        if (get_option('multipages_show_import_notification') === 'true') {
            mpg_show_notification('success', 'Demo data was applied.');
            update_option('multipages_show_import_notification', 'false');
        }

        // if permalinks set as ?p=123
        if ( get_option( 'permalink_structure' ) === '' ) {
	        mpg_show_notification( 'danger', __( 'Permalink structure cannot be "Plain".  Please click here to switch to', 'mpg' ) . ' <a href="#" class="change-permalink-structure" target="_blank">' . __( '"Post name" permalink structure', 'mpg' ) . '</a> ' . __( 'in order for MPG to work properly.', 'mpg' ) );
        } ?>


        <h1 class="mt-5 mb-5"><?php _e('MPG: Multiple Pages Generator by Porthas', 'mpg'); ?></h1>

        <?php

        if (mpg_app()->is_plan('Pro', true)) { ?>

            <div><span class="pro-version"><?php _e('PRO version', 'mpg'); ?></span></div>

        <?php } else { ?>

            <div><span class="free-version"><?php _e('FREE version', 'mpg'); ?></span></div>

        <?php } ?>

        <p>
            <?php _e('Please, visit', 'mpg'); ?> <a
                    href="<?php echo admin_url('admin.php?page=multi-pages-welcome-screen') ?>">
                <?php _e('Welcome walkthrough', 'mpg'); ?></a> <?php _e('to get instructions', 'mpg') ?>
        </p>

        <div class="row">
            <div class="col-sm-12 col-md-6">
                <div class="plugin-overview">

                    <p><?php _e('This plugin allows you to quickly create an infinite number of new pages on your site, using only import of settings for these pages from the .CSV table.', 'mpg') ?></p>
                    <p><?php _e('To get started, do these simple steps:', 'mpg') ?></p>
                    <ul>
                        <li>1) <?php _e('Select page as template.', 'mpg'); ?></li>
                        <li>2) <?php _e('Upload the .csv file.', 'mpg') ?></li>
                    </ul>
                </div>
            </div>

            <div class="col-sm-12 col-md-6">
                <?php
                if(!mpg_app()->is_plan('Pro', true)){ ?>

                    <section class="header-section">
                        <h2 class="font-weight-bold"><?php _e('Awesome Premium Features', 'mpg'); ?></h2>

                        <p><?php _e('Enjoy unlimited pages and many advanced features only available in PRO version.', 'mpg'); ?>
                            <a href="https://www.mpgwp.com/features/"
                               target="_blank"><?php _e('View Features', 'mpg') ?></a>
                        </p>
                        <a href="<?php echo mpg_app()->get_upgrade_url(); ?>">
                            <button class="btn btn-outline-primary"><?php _e('Upgrade to PRO', 'mpg'); ?></button>
                        </a>
                    </section>

                <?php } ?>
            </div>
        </div>

        <div class="modal fade" id="create-project">
            <div class="modal-dialog">
                <div class="modal-content">
                    <div class="modal-header">

                        <h6 class="modal-title"><?php _e('Enter your project name', 'mpg'); ?></h6>
                        <button type="button" class="close" data-dismiss="modal">&times;</button>
                    </div>
                    <div class="modal-body">
                        <div class="input-group">
                            <input type="text" class="project-name" required>
                            <button class="ml-2 create-project-button btn btn-primary"></button>
                        </div>
                    </div>
                </div>
            </div>
        </div>

        <div class="row ml-1 mr-1">
            <ul class="nav nav-tabs col-sm-11" role="tablist" id="project-tabs">
                <!-- Dynamic tab data will be pasted here-->
                <li>
                    <a href="#create_project" class="nav-link control-icon create" data-toggle="modal">
                        <img src="<?php echo MULTIPAGE_PLUGIN_URL; ?>/img/add.svg" alt="">
                    </a>
                </li>
            </ul>

            <div class="col-sm-1">
                <a href="admin.php?page=multi-pages-wizard" class="btn btn-success disabled wizard-go"><?php _e('Wizard', 'mpg');?></a>
            </div>
        </div>

        <div class="row template-section">
            <div class="col-sm-12 col-md-8 align-top">
                <form method="post" enctype="multipart/form-data" class="template-post">
                    <input type="hidden" value="" name="project-id">
                    <select name="post_template_id">
                        <?php $allPosts = get_pages();
                        foreach ($allPosts as $postData) { ?>

                            <option value="<?php echo $postData->ID; ?>"><?php echo $postData->post_title; ?></option>

                        <?php } ?>
                    </select>

                    <input type="submit" name="set_template" value="<?php _e('Set page template', 'mpg'); ?>"
                           class="btn btn-primary">
                </form>
            </div>

            <div class="col-sm-12 col-md-4">
                <div><?php _e('Current template post:', 'mpg'); ?></div>
                <div class="current-template-name"><?php _e('Loading...', 'mpg'); ?></div>
                <div class="current-template-id">ID: <span><?php _e('Loading...', 'mpg'); ?></span></div>
            </div>
        </div>

        <ul class="nav nav-tabs tab-section" role="tablist">
            <li class="nav-item">
                <a class="nav-link active" data-toggle="tab" href="#upload-template" role="tab"
                   aria-selected="true"><?php _e('Upload template', 'mpg'); ?></a>
            </li>
            <li class="nav-item">
                <a class="nav-link" data-toggle="tab" href="#schedule-template" role="tab"
                   aria-selected="false"><?php _e('Schedule', 'mpg'); ?></a>
            </li>
        </ul>
        <div class="tab-content">

            <div class="tab-pane upload-template fade show active" id="upload-template">

                <form method="post" enctype="multipart/form-data" class="template-file">
                    <div class="row">

                        <input type="hidden" name="project_id" value="">
                        <input type="hidden" name="action" value="mpg_data_source_upload_file">

                        <div class="col-sm-12 col-md-4">

                            <div class="custom-file">
                                <label class="custom-file-label"
                                       for="template-file"><?php _e('Choose file', 'mpg'); ?></label>
                                <input type="file" required name="multipages_csv_settings" accept=".csv"
                                       class="custom-file-input" id="template-file">
                            </div>
                        </div>

                        <div class="col-sm-12 col-md-4 text-center">

                            <div class="form-check form-check-inline p-md-2">
                                <input class="form-check-input" id="sitemap_settings_on" name="create_sitemap"
                                       type="checkbox"
                                       value="1">
                                <label class="form-check-label"
                                       for="sitemap_settings_on"><?php _e('Create sitemap file', 'mpg'); ?></label>
                            </div>
                        </div>

                        <div class="col-sm-12 col-md-4">
                            <input type="submit" name="upload_csv_settings"
                                   value="<?php _e('Upload settings', 'mpg'); ?>" class="btn btn-primary">
                        </div>
                    </div>

                    <div id="sitemap_settings">
                        <?php require_once('html/components/sitemap_configurator.php'); ?>
                    </div>
                </form>

                <div class="row shortcodes-description">
                    <div class="col-sm-12 col-md-8">
                        <p><?php _e('The column\'s header names of your uploaded file are the shortcodes for the front end template page. Simply copy the code, for example', 'mpg'); ?>
                            <b>{{mpg_headername}}</b>, <?php _e('and paste it where you want that data to appear on the new pages.', 'mpg'); ?>
                        </p>
                    </div>


                    <div class="col-sm-12 col-md-4">
                        <form class="form-inline search-form" method="post"
                              action="<?php echo admin_url("?page=multi-pages-plugin"); ?>">
                            <input class="form-control" name="mpp_search" type="search"
                                   placeholder="<?php _e('Type here...', 'mpg'); ?>" aria-label="Search">
                            <input type="submit" value="<?php _e('Search', 'mpg'); ?>"
                                   class="btn btn-outline-primary ml-2">
                        </form>
                    </div>
                </div>

                <div class="row search-block">
                    <div class="col-sm-12 col-md-8">
                        <p style="float: left;"><?php _e('Current CSV file:', 'mpg'); ?></p>
                    </div>
                    <div class="search-result col-sm-12 col-md-4"></div>
                </div>

                <div class="user-data"></div>

            </div>
            <div class="tab-pane schedule-template fade" id="schedule-template">

                <?php
                if (!mpg_app()->is_plan('Pro', true)) {
                    mpg_show_notification('warning', __('Schedule is a PRO feature.', 'mpg') . ' <a href="' . mpg_app()->get_upgrade_url() . '" target="_blank">' . __('Upgrade now', 'mpg') . '</a>');
                }
                ?>

                <p class="mt-3"><?php _e('Use schedule to periodically import your CSV file or Google Sheets. Just update your spreadsheets and new data will push into MPG on next import and update/create/delete all pages accordingly.', 'mpg'); ?></p>

                <form method="post" autocomplete="off" class="schedule-template-form">

                    <?php $disabled_status = mpg_app()->is_plan('Pro', true) ? '' : 'disabled="true"' ?>

                    <input type="hidden" name="mpg_timezone_name" value="">
                    <input type="hidden" name="mpg_project_id" value="">

                    <div class="row">
                        <div class="form-group col-sm-12">
                            <label for=""><?php _e('Direct link to .csv file, or shared Google Sheet', 'mpg'); ?></label>
                            <input name="remote_csv_link" class="form-control"
                                   type="url" <?php echo $disabled_status; ?> required="required">
                        </div>
                    </div>

                    <div class="row">
                        <div class="col-sm-12 col-md-6">
                            <label><?php _e('Set periodicity', 'mpg'); ?></label>
                            <select name="periodicity" <?php echo $disabled_status; ?> class="form-control"
                                    style="height: 38px;">
                                <option value="once"><?php _e('Once', 'mpg'); ?></option>
                                <option value="hourly"><?php _e('Hourly', 'mpg'); ?></option>
                                <option value="twicedaily"><?php _e('Twice per day', 'mpg'); ?></option>
                                <option value="daily"><?php _e('Daily', 'mpg'); ?></option>
                                <option value="weekly"><?php _e('Weekly', 'mpg'); ?></option>
                                <option value="monthly"><?php _e('Monthly', 'mpg'); ?></option>
                            </select>
                        </div>

                        <div class="form-group col-sm-12 col-md-6">
                            <label><?php _e('Set applying datetime', 'mpg'); ?></label>
                            <input name="datetime_upload_remote_csv" class="form-control"
                                   required="required" <?php echo $disabled_status; ?> type="text">
                        </div>
                    </div>

                    <div class="row">
                        <div class="col-sm-12 col-md-6">
                            <label for="notification_email"><?php _e('Set Email address for notifications', 'mpg'); ?></label>
                            <input name="notification-email" <?php echo $disabled_status; ?> class="form-control"
                                   id="notification_email" type="email"
                                   value="<?php echo get_option('admin_email'); ?>">
                        </div>
                        <div class="col-sm-12 col-md-6 mt-3">
                            <span><?php _e('Notification:', 'mpg'); ?></span>
                            <div class="form-group">
                                <div class="form-check form-check-inline">
                                    <input class="form-check-input" <?php echo $disabled_status; ?> type="radio"
                                           checked="checked" name="cron-notification"
                                           id="do-not-notify" value="do-not-notify">
                                    <label class="form-check-label"
                                           for="do-not-notify"><?php _e('Do not notify', 'mpg'); ?></label>
                                </div>
                                <div class="form-check form-check-inline">
                                    <input class="form-check-input" <?php echo $disabled_status; ?> type="radio"
                                           name="cron-notification" id="errors-only"
                                           value="errors-only">
                                    <label class="form-check-label"
                                           for="errors-only"><?php _e('Errors only', 'mpg'); ?></label>
                                </div>
                                <div class="form-check form-check-inline">
                                    <input class="form-check-input" <?php echo $disabled_status; ?> type="radio"
                                           name="cron-notification" id="every-time"
                                           value="every-time">
                                    <label class="form-check-label"
                                           for="every-time"><?php _e('Every time', 'mpg'); ?></label>
                                </div>
                            </div>
                        </div>
                    </div>

                    <div class="row">
                        <div class="col-sm-12 col-md-2">
                            <input type="submit" name="action"
                                   value="<?php _e('Schedule', 'mpg'); ?>" <?php echo $disabled_status; ?>
                                   class="btn btn-primary mt-4">
                        </div>
                    </div>
                </form>

                <form method="post" autocomplete="off" class="unschedule-template-form">

                    <input type="hidden" name="mpg_project_id" value="">

                    <div class="row">
                        <div class="col-sm-12 col-md-6">
                            <input name="datetime_upload_remote_csv" type="text" class="form-control"
                                   disabled="disabled" value="">
                        </div>
                        <div class="col-sm-12 col-md-6">
                            <input type="submit" name="action" value="<?php _e('Unschedule', 'mpg'); ?>"
                                   class="btn btn-danger">
                        </div>
                    </div>

                </form>
            </div>
        </div>



        <!-- The Modal -->
        <div class="modal fade" id="url-builder-modal">
            <div class="modal-dialog modal-dialog-centered modal-lg">
                <div class="modal-content">
                    <div class="url-builder-dashboard-modal">
                        <p>This is columns names of your dataset. Click to this shortcode for adding to url builder:</p>

                        <div class="row"><div class="shortcodes-pills mb-4 pb-3"></div> </div>
                        <div class="row">
                            <div class="url-builder-block mt-5 mb-5">
                                <span>/</span>
                                <div class="url-builder-input" contenteditable="true"></div>
                                <span>/</span>
                            </div>
                        </div>

                        <div class="row mt-5">
                            <div class="col-sm-12 col-md-6"><p>So, finally, URL template will look like this: </p><p class="result-url"></p></div>
                            <div class="col-sm-12 col-md-6">
                                <div class="row">
                                    <div class="col-sm-12 col-md-12">
                                        <p>Replace spaces in column to:</p>

                                        <select name="" id="" class="space-replacer-select">
                                            <option value="-">- (dash)</option>
                                            <option value="_">_ (low dash)</option>
                                            <option value="slash">/ (slash)</option>
                                        </select>
                                    </div>
                                </div>
                                <small>For example, if you choose dash, as a replacer, "New Orleans" will be replaced to New-Orleans in your dataset</small>
                            </div>
                        </div>
                        <div class="navigator">
                            <button class="btn btn-info cancel">Cancel</button>
                            <button class="btn btn-success next disabled">Next</button>
                        </div>
                    </div>

                </div>
            </div>
        </div>

    </div>
    <?php
}
