<?php

function mpg_wizard() {
	require_once( 'template.php' );
}

add_action( 'wp_ajax_mpg_data_source_upload_file', 'mpg_data_source_upload_file' );

/**
 * Handle user choice of data source: file, remote csv, or dataset.
 */

function mpg_data_source_upload_file() {
	$path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/';

	if ( ! empty( $_FILES['multipages_csv_settings'] && $_POST['project_id'] ) ) {

        try {

			$filename      = $_FILES['multipages_csv_settings']['name'];
			$project_id    = (int) $_POST['project_id'];
			$temp_filename = $_FILES['multipages_csv_settings']['tmp_name'];

			$ext = strtolower( pathinfo( $filename, PATHINFO_EXTENSION ) );

			$final_data_file = getSiteName() . '-' . $project_id . '-settings.json';

			if ( $ext === 'csv' ) {

				$destination = $path . $final_data_file;

				if ( move_uploaded_file( $temp_filename, $destination ) ) {

					$csv_file_content = file_get_contents( $destination );

					if ( ! $csv_file_content ) {
						throw new Exception( __( 'Can not get content of uploaded file.', 'mpg' ) );
					}

					// Check users csv to correct encoding and values separator.
					$validation_results = mpg_csv_validator($csv_file_content);

					$user_data_array = csv_to_array( $csv_file_content );

					if ( ! $user_data_array ) {
						throw new Exception( __( 'Can not convert user .csv to .json', 'mpg' ) );
					}

					$shortcodes_array = generate_shortcodes_array( $user_data_array );

					if ( $shortcodes_array['url_column_index'] === false ) {
						// It's mean, that users .csv haven't URL column.

						$response = [
							'success' => true,
							'columns' => implode( ',', $shortcodes_array['shortcodes_array'] )
						];

						if(count($validation_results)){
							$response['validation_errors'] = $validation_results;
						}

						echo json_encode( $response );

					} else {
						echo user_data_processing($user_data_array, $project_id );
					}

				} else {
					throw new Exception( __( 'Can not move file from temp directory. Check permissions.', 'mpg' ) );
				}

			} else {
				throw new Exception( __( 'Invalid file extension', 'mpg' ) );
			}
		} catch ( Exception $e ) {
			echo json_encode( [ 'success' => false, 'notifications' => $e->getMessage() ] );
		}
	}

	wp_die();
}


add_action( 'wp_ajax_mpg_data_source_remote_file', 'mpg_data_source_remote_file' );


function mpg_data_source_remote_file() {

	$project_id      = (int) $_POST['projectId'];
	$remote_file_url = trim( strip_tags( esc_sql( $_POST['remoteFileUrl'] ) ) );
	try {
		if ( $project_id && $remote_file_url ) {

			$link             = get_direct_csv_link( $remote_file_url );
			$csv_file_content = file_get_contents( $link );

			if ( ! $csv_file_content ) {
				throw new Exception( __( 'Can not get content of uploaded file.', 'mpg' ) );
			}

			// Check users csv to correct encoding and values separator.
			$validation_results = mpg_csv_validator($csv_file_content);

			if(count($validation_results)){
			
				echo json_encode([
					'success' => true,
					'validation_errors' => $validation_results
				]);

				wp_die();
			}

			$user_json = csv_to_array( $csv_file_content );

			if ( ! $user_json ) {
				throw new Exception( __( 'Can not convert user .csv to .json', 'mpg' ) );
			}

			$shortcodes_array = generate_shortcodes_array( $user_json );

			if ( $shortcodes_array['url_column_index'] === false ) {
				// It's mean, that users .csv haven't URL column.

				$data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';
				// set dynamic link with project id
				$user_data_file_resource = fopen( $data_file_path, 'w+' );

				if ( ! fwrite( $user_data_file_resource, $csv_file_content ) ) {
					throw new Exception( __( 'Can not write your data to file. Please, check permission' ), 'mpg' );
				}

				echo json_encode( [
					'success' => true,
					'columns' => implode( ',', $shortcodes_array['shortcodes_array'] )
				] );

			} else {
				echo user_data_processing( $user_json, $project_id );
			}

		} else {
			throw new Exception( __( 'Remote file URL is missing.', 'mpg' ) );
		}

	} catch ( Exception $e ) {
		echo json_encode( [ 'success' => false, 'notifications' => $e->getMessage() ] );
	}

	wp_die();
}


add_action( 'wp_ajax_mpg_data_source_dataset', 'mpg_data_source_dataset' );


function mpg_data_source_dataset() {
	$project_id       = (int) $_POST['projectId'];
	$selected_dataset = esc_sql( $_POST['selectedDataset'] );

	try {

		if ( $project_id  ) {

				$selected_dataset_path = realpath( plugin_dir_path( __FILE__ ) . '../../datasets/' . $selected_dataset . '.csv' );

				$open_resource = fopen( $selected_dataset_path, 'r' );

				if ( ! $open_resource ) {
					throw new Exception( __( 'Can not to open dataset .csv file', 'mpg' ) );
				}

				$dataset_content = fread( $open_resource, filesize( $selected_dataset_path ) );

				if ( ! $dataset_content ) {
					throw new Exception( __( 'Can not to read dataset file', 'mpg' ) );
				}

				$dataset_content_array = csv_to_array( $dataset_content );

				if ( ! $dataset_content_array ) {
					throw new Exception( __( 'Can not to decode json of dataset file', 'mpg' ) );
				}

				$shortcodes_array = generate_shortcodes_array( $dataset_content_array );

				if ( $shortcodes_array['url_column_index'] === false ) {
					// It's mean, that users .csv haven't URL column.

					$data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';
					// set dynamic link with project id
					$user_data_file_resource = fopen( $data_file_path, 'w+' );

					if ( ! fwrite( $user_data_file_resource, $dataset_content ) ) {
						throw new Exception( __( 'Can not write your data to file. Please, check permission' ), 'mpg' );
					}

					echo json_encode( [
						'success' => true,
						'columns' => implode( ',', $shortcodes_array['shortcodes_array'] )
					] );

				} else {
					echo user_data_processing( $dataset_content_array, $project_id );
				}
				wp_die();

		} else {
			throw new Exception( __( 'Missing project id', 'mpg' ) );
		}

	} catch ( Exception $exception ) {
		echo json_encode( [ 'success' => false, 'notifications' => $exception->getMessage() ] );
		wp_die();
	}
}

add_action('wp_ajax_mpg_generate_sitemap_to_project', 'mpg_generate_sitemap_to_project');

function mpg_generate_sitemap_to_project(){

	$project_id            = isset($_POST['projectId'])           ? (int) $_POST['projectId'] : null;
	$sitemap_name          = isset($_POST['sitemap_name'])        ? esc_sql($_POST['sitemap_name']) : null;
	$sitemap_max_url       = isset($_POST['sitemap_max_url'])     ? esc_sql($_POST['sitemap_max_url']) : null;
	$sitemap_update_freq   = isset($_POST['sitemap_update_freq']) ? esc_sql($_POST['sitemap_update_freq']) : null;
	$add_to_robots         = isset($_POST['add_to_robots'])       ? esc_sql($_POST['add_to_robots']) : null;

	try {
		$urls_list_raw_array = null;

		$project_object = get_option('multipages_project_' . $project_id);

		if($project_object){
			$urls_list_raw_array = $project_object['csv_settings_urls'];
		}

		$url_list = json_decode($urls_list_raw_array, true);

		if ( ! mpg_sitemap_generator(array_keys($url_list), $sitemap_name, $sitemap_max_url, $sitemap_update_freq, $add_to_robots) ) {
			throw new Exception(__('Can not generate sitemap file.', 'mpg'));
		}

		$sitemap_filename = $sitemap_name ? $sitemap_name . '.xml' : 'multipage-sitemap.xml';

		$link =  getDomain() . str_replace('//', '/', '/' . getSiteUrl() . '/' . $sitemap_filename);
		// replace need to be rewrited;

		echo json_encode(['success' => true, 'link' => $link]);
		wp_die();
	}catch (Exception $exception){
		echo json_encode(['success' => false, 'notifications' => $exception->getMessage()]);
		wp_die();
	}
}

add_action( 'wp_ajax_mpg_create_url_for_dataset', 'mpg_create_url_for_dataset' );

function mpg_create_url_for_dataset() {
	$project_id            = (int) $_POST['projectId'];
	$url_template          = esc_sql( $_POST['urlTemplate'] );
	$is_needed_sitemap     = isset($_POST['isNeededSitemap']) ? $_POST['isNeededSitemap']  : null;
	$sitemap_name          = isset($_POST['sitemap_name']) ? esc_sql($_POST['sitemap_name']) : null;
	$sitemap_max_url       = isset($_POST['sitemap_max_url']) ? esc_sql($_POST['sitemap_max_url']) : null;
	$sitemap_update_freq   = isset($_POST['sitemap_update_freq']) ? esc_sql($_POST['sitemap_update_freq']) : null;
	$add_to_robots         = isset($_POST['add_to_robots']) ? esc_sql($_POST['add_to_robots']) : null;

	try {
		if ( $project_id && $url_template ) {
			$data_file_path = MULTIPAGE_PLUGIN_PATH . 'uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';

			$file_resource_read = fopen( $data_file_path, 'r' );
			if ( ! $file_resource_read ) {
				throw new Exception( __( 'Error opening dataset file.', 'mpg' ) );
			}
			$dataset_without_url = fread( $file_resource_read, filesize( $data_file_path ) );
			if ( ! $dataset_without_url ) {
				throw new Exception( __( 'Can not to read dataset file.', 'mog' ) );
			}
			$csv_dataset = csv_to_array( $dataset_without_url );
			// Cut headers to separate array.
			$headers = array_shift( $csv_dataset );
			array_unshift($headers);
			$shortcodes_storage = [];
			foreach ( $headers as $index => $header_name ) {
				// Iterating headers form array. And concat prefix, for finding.
				// Because from frontend come header name without prefix.
				$header_shortcode     = 'mpg_' . $header_name;
				$shortcodes_storage[] = $header_shortcode;
			}

			$sitemap_urls_list = [];
			$dataset_with_url_column = [];
			// Iterate every row of dataset
			foreach ( $csv_dataset as $dataset_row ) {

				$url = trim(str_replace( $shortcodes_storage, $dataset_row, $url_template ));

				$url = str_replace([' '], esc_sql( $_POST['spaceReplacer']) , strtolower($url));
				$url = str_replace(PHP_EOL, '' , strtolower($url));

				$sitemap_urls_list[] = $url;
				// And merge incoming url value with string from user dataset.
				$dataset_with_url_column[] = array_merge( [ $url ], $dataset_row );
			}
			// Remove "mpg" from headers names.
			$shortcodes_storage = array_map( function($str) {
				return str_replace('mpg_', '', $str);}, $shortcodes_storage );
			array_unshift($shortcodes_storage, 'url');


			$final_dataset_with_url = array_merge( [ $shortcodes_storage ], $dataset_with_url_column );
			$data_processing_result = json_decode(user_data_processing($final_dataset_with_url, $project_id), true);


			if ($is_needed_sitemap === "true"  and  class_exists( 'MPG_SitemapGenerator' ) ) {
				if(!mpg_sitemap_generator( $sitemap_urls_list, $sitemap_name, $sitemap_max_url,  $sitemap_update_freq, $add_to_robots )){
					throw new Exception(__('Can not to create sitemap. Unknown error occurred.'));
				}

				$sitemap_filename = $sitemap_name ? $sitemap_name . '.xml' : 'multipage-sitemap.xml';
				$data_processing_result['notifications'][] = ['type' => 'success', 'text' => __('Sitemap was created. ', 'mpg') .  getDomain() . '/' . getSiteUrl() . '/' . $sitemap_filename];
			}

			echo json_encode($data_processing_result);
			wp_die();
		}
	} catch ( Exception $exception ) {
		echo json_encode( [ 'success' => false, 'notifications' => $exception->getMessage() ] );
		wp_die();
	}
}

add_action( 'wp_ajax_mpg_get_available_pages', 'mpg_get_available_pages' );

function mpg_get_available_pages() {

	try {
		$all_pages = get_pages();
		$storage   = [];

		foreach ( $all_pages as $page ) {
			$storage[ $page->ID ] = $page->post_title;
		}

		echo json_encode( [ 'success' => true, 'data' => $storage ] );
		wp_die();

	} catch ( Exception $e ) {
		echo json_encode( [ 'success' => false, 'notifications' => $e->getMessage() ] );
	}
}

add_action( 'wp_ajax_mpg_get_headers_by_project_id', 'mpg_get_headers_by_project_id' );

function mpg_get_headers_by_project_id() {

	$project_id = (int) $_POST['projectId'];

	try {

		if ( $project_id ) {

			$project_object = get_option( 'multipages_project_' . $project_id );

			if ( ! $project_object ) {
				throw new Exception( __( 'Project with specified id was not found', 'mpg' ) );
			}

			if ( ! isset( $project_object['csv_settings_shortcodes'] ) ) {
				throw new Exception( __( 'Dataset not uploaded yet. Please, upload dataset from .csv or Google Sheets.', 'mpg' ) );
			}

			$rawShortcodes = $project_object['csv_settings_shortcodes'];

			$storage         = [];
			$shortcodesArray = json_decode( $rawShortcodes, true );
			foreach ( $shortcodesArray as $shortcode ) {
				$storage[] = str_replace( 'mpg_', '', $shortcode );
			}

			echo json_encode( [ 'success' => true, 'shortcodes' => $storage ] );
			wp_die();
		}
	} catch ( Exception $exception ) {

		echo json_encode( [ 'success'       => false,
		                    'notifications' => [ 'type' => 'danger', 'text' => $exception->getMessage() ]
		] );
		wp_die();
	}
}