import {init, toggleActiveSections} from "./project_tab.js";
import {__} from '../../languages/loader.js';
import {renderNotification} from "../../helpers.js";

let createProject = $('#create-project');

let modalTitle = createProject.find('.modal-title');
let projectNameInput = createProject.find('.modal-body .project-name');
let submitButton = createProject.find('.modal-body button');


$(document).on('click', '.control-icon', function () {


    let activeTab = $('#project-tabs .nav-link.active.show');
    projectNameInput.trigger('focus');

    if ($(this).hasClass('create')) {

        modalTitle.text(__('Enter your project name'));

        submitButton
            .text(__('Create'))
            .removeClass('btn-danger create rename delete')
            .addClass('btn-primary create');

        projectNameInput
            .val('')
            .removeAttr('disabled');

    } else if ($(this).hasClass('rename')) {

        // Set input value in modal, as name of active tab.
        projectNameInput.val(activeTab.text().trim());

        projectNameInput.removeAttr('disabled');

        submitButton
            .removeClass('btn-danger create rename delete')
            .addClass('btn-primary rename')
            .text(__('Rename'));

    } else if ($(this).hasClass('delete')) {

        if ($('#project-tabs .nav-link[role="tab"]').length === 1) {

            renderNotification(
                [{
                    text: __('You can not delete latest project'),
                    type: 'danger'
                }]);

            return;
        }

        // Set input value as name of active tab.
        projectNameInput.val(activeTab.text().trim());

        projectNameInput.attr('disabled', 'disabled');

        submitButton
            .removeClass('btn-primary create rename delete')
            .addClass('btn-danger delete')
            .text(__('Delete'));

        modalTitle.text(__('Are you sure want to delete project with all data? Please, notice that you can not undo this operation'));
    }

    // Open modal
    createProject.modal('show');

});

function deleteProject(projectId, activeTab){

    $.post(
        ajaxurl,
        {
            action: 'mpg_delete_project',
            project_id: projectId ? projectId : localStorage.getItem('mpg_current_tab_id')

        }, function (response) {

            if (JSON.parse(response).success === true) {
                $('#create-project').modal("hide");

                // Set active to sibling tab
                let tabsLinks = $('#project-tabs .nav-link[role="tab"]');
                let tabsLength = tabsLinks.length;
                let activeTabNumber = 0;
                let futureActiveTab;

                $.each(tabsLinks, function (index, elem) {
                    index++; // lets count from 1

                    if ($(elem).hasClass('active')) {
                        activeTabNumber = index;
                    }
                });

                if (activeTabNumber === tabsLength) {
                    // If this tab is latest
                    futureActiveTab = $(tabsLinks[activeTabNumber - 2]);
                    futureActiveTab.addClass('active show');

                } else {
                    futureActiveTab = $(tabsLinks[activeTabNumber]);
                    futureActiveTab.addClass('active show');
                }

                if (futureActiveTab.attr('href')) {
                    localStorage.setItem('mpg_current_tab_id', futureActiveTab.attr('href').replace('#', ''));
                }

                activeTab ? activeTab.remove() : null;

                projectNameInput.val('').attr('disabled', null); // clear input

            } else {
                renderNotification([{type: 'danger', text: response}])
            }
        })
        .fail((error) => {
            renderNotification([{type: 'danger', text: error}])
        });
}


submitButton.on('click', function () {

    let activeTab = $('#project-tabs .nav-link.active.show');

    if ($(this).hasClass('create')) {

        let projectName = $('.project-name').val();
        const min = 111111;
        const max = 9999999;
        let randomId = Math.round(Math.random() * (max - min) + min);
        $.post(
            ajaxurl,
            {
                action: 'mpg_create_project',
                project_name: projectName,
                id: randomId

            }, function (response) {

                if (JSON.parse(response).success === true) {
                    createProject.modal("hide");

                    localStorage.setItem('mpg_current_tab_id', randomId.toString());

                    // Start getting data by this project.
                    init(randomId);

                } else {
                    renderNotification([{type: 'danger', text: error}]);
                }
            })
            .fail((error) => {
                renderNotification([{type: 'danger', text: error}]);
            });

    } else if ($(this).hasClass('rename')) {

        $.post(
            ajaxurl,
            {
                action: 'mpg_rename_project',
                project_name: projectNameInput.val(),
                project_id: localStorage.getItem('mpg_current_tab_id')

            }, function (response) {

                if (JSON.parse(response).success === true) {
                    $('#create-project').modal("hide");
                    // Set new name to tab
                    activeTab.find('span').text(projectNameInput.val());

                    projectNameInput.val(''); // clear input

                } else {
                    renderNotification([{type: 'danger', text: JSON.parse(response)}]);
                }
            })
            .fail((error) => {
                renderNotification([{type: 'danger', text: error}]);
            });

    } else if ($(this).hasClass('delete')) {

        deleteProject(null, activeTab);

    }
});

export {deleteProject};