// Save to localStorage last opened document id

import { __ } from '../../languages/loader.js';
import { renderNotification, stringEndsWith } from "../../helpers.js";
import { renderUrlBuilderModal } from "../wizard/functions.js";

$ = jQuery;
$(document).on('click', '#project-tabs .nav-item a', function () {

    let currentTabId = $(this).attr('href').replace('#', '');
    localStorage.setItem('mpg_current_tab_id', currentTabId);

    // After click to project tab - show "Upload template" tab. Maybe users has opened "Schedule" ?
    $('a[href="#upload-template"]').trigger('click');

    $('.user-data').hide();
    $('.schedule-template-form').hide();
    $('.pagination-block').hide();

    init(currentTabId);

});


function toggleActiveSections(tabLength) {

    let activeSectionsSelector = $('.tab-content, .tab-section, .template-section');

    if (tabLength) {
        activeSectionsSelector.show();
    } else {
        activeSectionsSelector.hide();
    }
}

function getTabs(currentTabId) {

    let projectTabs = $('#project-tabs');

    $.post(
        ajaxurl,
        {
            action: 'mpg_get_tabs',
        }, function (response) {
            if (response) {
                let responseObject = JSON.parse(response);
                let tabData = responseObject.data;
                let tabsIdArray = [];

                let tabs = '';

                for (let index in tabData) {
                    let tabInfo = tabData[index];

                    tabsIdArray.push(tabInfo.id);

                    tabs += `<li class="nav-item">
                        <a  class="nav-link" data-toggle="tab" href="#${tabInfo.id}" role="tab" 
                            aria-controls="upload-template" aria-selected="true">
                            <span>${tabInfo.name}</span>
                        </a>
                         <img class="control-icon rename" src="${responseObject.plugin_base_url}img/edit.svg" alt="">
                         <img class="control-icon delete" src="${responseObject.plugin_base_url}img/clear.svg" alt="">
                    </li>`;
                }

                // Remove "old" tabs
                projectTabs.find('li.nav-item').remove();
                // Insert new tabs list
                projectTabs.prepend(tabs);
                // Set active tab
                projectTabs.find(`.nav-item a[href="#${currentTabId}"]`).addClass('active show');

                $('input[name="project-id"]').val(currentTabId);
                // Get data for last opened tab

                // MultiTab is a PRO feature. In FREE available only one tab;
                if (!responseObject.is_pro_user) {

                    $('.mpg-dashboard .wizard-go').addClass('disabled');

                    let setTemplateControl = $('.template-post input[name="set_template"], .template-post select[name="post_template_id"]');

                    $('.template-section div.alert.alert-warning').remove();

                    setTemplateControl.removeAttr('disabled')

                    let currentTabId = localStorage.getItem('mpg_current_tab_id');

                    tabsIdArray.shift();

                    if (tabsIdArray.includes(currentTabId)) {

                        setTemplateControl.attr('disabled', 'disabled');
                        // todo: change to render notification
                        $('.template-section').prepend(`<div class="alert alert-warning show" style="width:99%">
                            ${__('Multi-Template is a PRO feature.')} <a href="${responseObject.upgrade_url}" target="_blank">${__('Upgrade now')}</a></div>`);
                    }
                } else {

                    $('.mpg-dashboard .wizard-go').removeClass('disabled');

                }
            }
        });
}

function getProjectMetaInfo(currentTabId) {

    return new Promise((resolve, reject) => {

        $.post(
            ajaxurl,
            {
                action: 'mpg_get_project_info',
                projectId: currentTabId
            }, function (response) {
                if (response) {
                    let responseObject = JSON.parse(response);

                    // Template name
                    let currentTemplateName = $('.current-template-name');
                    if (responseObject.data && responseObject.data.template_name) {
                        currentTemplateName.text(responseObject.data.template_name);
                    } else {
                        currentTemplateName.text('Not set');
                    }

                    // Template ID
                    let currentTemplateId = $('.current-template-id span');
                    let uploadDataFileInput = $('.template-file input[name="multipages_csv_settings"]');
                    let createSitemapCheckbox = $('.template-file input[name="create_sitemap"]');
                    let submitDataFileButton = $('.template-file input[name="upload_csv_settings"]');
                    let searchFormInputs = $('.search-form input');
                    let scheduleTab = $('a[href="#schedule-template"]');
                    let importDemoBtn = $('.activate_demo_data');

                    if (responseObject.data && responseObject.data.template_id) {
                        currentTemplateId.text(responseObject.data.template_id);
                        uploadDataFileInput.removeAttr('disabled');
                        createSitemapCheckbox.removeAttr('disabled');
                        submitDataFileButton.removeAttr('disabled');
                        searchFormInputs.removeAttr('disabled');
                        // scheduleTab.removeClass('disabled');
                        importDemoBtn.removeClass('disabled');
                        // Set value in dropdown
                        $('select[name="post_template_id"]').val(responseObject.data.template_id);

                    } else {

                        currentTemplateId.text('Not set');
                        uploadDataFileInput.attr('disabled', 'true');
                        createSitemapCheckbox.attr('disabled', 'true');
                        submitDataFileButton.attr('disabled', 'true');
                        searchFormInputs.attr('disabled', 'true');
                        // scheduleTab.addClass('disabled');
                        importDemoBtn.addClass('disabled');
                        $('select[name="post_template_id"]').prop('selectedIndex', 0);
                    }

                    // Remove checkbox for "Create sitemap";
                    $('#sitemap_settings_on').prop('checked', false);
                    // Hide sitemap configurator for case if it's was opened;
                    $('#sitemap_settings').hide('fast');
                    // For data file upload template form need to set project id.
                    $('input[name="project_id"]').val(localStorage.getItem('mpg_current_tab_id'));

                    resolve();
                    // todo: no needed to execute code below, if this form is in wizard;
                    $('form.template-file').unbind('submit').bind('submit', function (e) {
                        e.preventDefault();

                        let t = this;
                        $.ajax({
                            url: ajaxurl,
                            type: "POST",
                            data: new FormData(t),
                            contentType: false,
                            cache: false,
                            processData: false,
                            success: function (rawResponse) {

                                console.log('this is thame callback')

                                let response = JSON.parse(rawResponse);

                                if (response.success === true) {

                                    if (response.validation_errors) {
                                        response.validation_errors.forEach(errorText => {
                                            renderNotification([{ type: 'warning', text: errorText }]);
                                        })
                                    }

                                    let columnList = response.columns;

                                    if (response.success && !columnList) {
                                        getProjectUserData(currentTabId, 1);
                                    } else if (columnList) {
                                        renderUrlBuilderModal(columnList);
                                    }
                                    renderNotification(response.notifications);
                                } else {
                                    renderNotification(response.notifications);
                                }
                            },
                            complete: function (xhr, textStatus) {
                                if (xhr.status === 400) {

                                    renderNotification([
                                        {
                                            type: 'warning',
                                            text: `If uploaded file size more than 8 Mb, uploading may fall. Please, update php.ini to increase "upload_max_filesize" and "post_max_size".
                                               For details, contact with your hosting provider`
                                        }]);
                                }
                            },
                        });
                    });
                }else{
                    reject('Can\'t get project info');
                }

                
            })
            .fail(function (error) {
                reject(JSON.stringify(error));
            });
    })
}

/**
 * @param responseObject
 * @param neededPage
 * @param context - may be "search" or "regular". Needed for correct work of pagination.
 */
function renderTableWithPagination(responseObject, neededPage, context) {

    $('.user-data').remove();
    $('.pagination-block').remove();
    $('.import-demo-block').remove();

    let shortcodes = '';
    for (let row in responseObject.shortcodes) {
        shortcodes += `<th scope="col" class="table-primary">${responseObject.shortcodes[row]}</th>`
    }

    let data = '';
    for (let row in responseObject.data) {

        data += '<tr>';
        responseObject.data[row].forEach(tableCeil => {
            data += `<td>${tableCeil}</td>`;
        });
        data += '</tr>';
    }

    let table =
        `<div class="user-data">
            <table class="table table-hover table-striped">
                  <thead>
                       <tr>${shortcodes}</tr>
                  </thead>
                  <tbody>${data}</tbody>
             </table>
        </div>`;

    let pager = '';
    let activeStatus = '';
    let lastStatus = '';
    let pagination = '';

    // Render pagination only if user data file is available
    if (responseObject['pages']) {

        for (let i = 1; i <= responseObject['pages']; i++) {
            if (i > neededPage - 2 && i < neededPage + 4) {

                i === responseObject['pages'] ? lastStatus = 'last' : '';
                pager += `<li class="page-item ${activeStatus}"><a class="page-link ${lastStatus}" href="#${i}">${i}</a></li>`;
            }
        }

        pagination =
            `<div class="row pagination-block" data-context="${context}">
                        <nav aria-label="Page navigation" class="center-block">
                            <ul class="pagination">
                                <li class="page-item">
                                    <a href="#" aria-label="Previous" class="page-link">&laquo;</a>
                                        <span class="sr-only">Previous</span>
                                    </a>
                                </li>
                                
                                ${pager}
                                
                                <li class="page-item">
                                    <a class="page-link" href="#" aria-label="Next">
                                        <span aria-hidden="true">&raquo;</span>
                                        <span class="sr-only">Next</span>
                                    </a>
                                </li>
                            </ul>
                        </nav>
                    </div>`;
    }

    $('#upload-template').append(table + pagination);

    if (neededPage > 0) {
        $('.pagination .page-link[href="#' + neededPage + '"]').parent().addClass('active');
    } else {
        // If no one page is not clicked - make 1 as active;
        $('.pagination li:nth-child(2)').addClass('active');
    }
}


function renderImportDemoBlock() {

    $('.user-data').remove();
    $('.pagination-block').remove();
    $('.import-demo-block').remove();

    let block = '<div class="import-demo-block">\n' +
        '          <p>' + __('No settings yet') + '<br>' +
        __('You can select your template page above and') + '</p>' +
        '          <button class="btn btn-primary activate_demo_data disabled">' + __('Import Demo Data') + '</button>' +
        '        </div>';

    $('#upload-template').append(block);

}

function getProjectUserData(currentTabId, page) {

    return new Promise((resolve, reject) => {

        // Datetime picker init
        let dateObject = new Date();
        $('input[name="datetime_upload_remote_csv"]').datetimepicker({
            minuteStepping: 1,               //set the minute stepping
            minDate: `1/1/1900`,
            minTime: `${dateObject.getHours()}:${dateObject.getMinutes()}`,
            step: 10 // minutes
        });

        $('input[name="mpg_timezone_name"]').val(Intl.DateTimeFormat().resolvedOptions().timeZone);

        let neededPage = page ? page : 1;

        $.post(
            ajaxurl,
            {
                action: 'mpg_get_data_file_content',
                projectId: currentTabId,
                page: neededPage

            }, function (response) {
                if (response) {
                    let responseObject = JSON.parse(response);

                    if (responseObject.data === null) {
                        console.log('render!')
                        renderImportDemoBlock();
                    } else {
                        renderTableWithPagination(responseObject, neededPage, 'regular');
                    }

                    if (responseObject.cron_execution_datetime) {
                        // If datetime is stored in DB - it's mean that project has a cron task
                        $('.schedule-template-form').hide();
                        $('.unschedule-template-form').show();
                        $('input[name="datetime_upload_remote_csv"]').val(responseObject.cron_execution_datetime);

                    } else {
                        $('.schedule-template-form').show();
                        $('.unschedule-template-form').hide();
                    }

                    if (responseObject.urls) {
                        let urlsList = Object.values(responseObject.urls);
                        if (urlsList) {

                            let finalUrl = baseUrl + Object.values(urlsList)[0];

                            $('.wizard .step-5 .example-url')
                                .html(`<a href="${finalUrl}" target="_blank">${finalUrl}</a>`);

                            $('.wizard .step-5 .example-url-block').show();
                        }
                    }

                    resolve();
                }

                reject('Can\'t get content of data file');
            });
    })
}

$('.change-permalink-structure').on('click', function (e) {
    e.preventDefault();
    $.post(
        ajaxurl,
        {
            action: 'mpg_change_permalink_structure',
        }, function (response) {
            if (response) {
                let responseObject = JSON.parse(response);

                if (responseObject.success === true) {
                    renderNotification([{ type: 'success', text: responseObject.notifications }]);
                } else {
                    renderNotification([{ type: 'warning', text: responseObject.notifications }]);
                }
            }
        });
});

async function init(currentTabId) {
    // с версии 2.8 'ajaxurl' всегда определен в админке

    if (currentTabId) {

        getTabs(currentTabId);

        try{
            
            await getProjectUserData(currentTabId);
            await getProjectMetaInfo(currentTabId);

        }catch(error){
            renderNotification([{ type: 'danger', text: error}]);
        }

        localStorage.setItem('mpg_wizard_project', JSON.stringify({ projectId: null }));
        $('input[name="mpg_project_id"]').val(localStorage.getItem('mpg_current_tab_id'));

    } else {

        toggleActiveSections(0);

    }
}


let currentTabId = localStorage.getItem('mpg_current_tab_id');

if (!currentTabId) {
    // Default tab always have id = 1. So, if it's a first installation - create default tab and make it active
    currentTabId = 1;
    localStorage.setItem('mpg_current_tab_id', currentTabId);
}

// Run function only in Dashboard. Not on Welcome Walkthrough or Wizard

if (stringEndsWith(location.href, 'multi-pages-plugin')) {
    init(currentTabId);
}


export { init, getProjectUserData, renderTableWithPagination, toggleActiveSections };