

// Wizard

    $ = jQuery;
    import {renderNotification, stringEndsWith} from "../../helpers.js";
    import {__} from "../../languages/loader.js";
    import {init, getProjectUserData} from "../dashboard/project_tab.js";
// Click to Next button on press Enter;


    $(document).on('keypress', function (event) {

        if (stringEndsWith(location.href, 'multi-pages-wizard')) {

            if (event.key === 'Enter') {

                event.preventDefault();

                let currentStep = parseInt(localStorage.getItem('mpgCurrentStep'));

                $(`.wizard .step-${currentStep} .next`).trigger('click');
            }
        }
    });




    function highlightBlock(thisSourceBlock, source) {
        source.removeClass('active');
        thisSourceBlock.addClass('active');
        $('.wizard .navigator .next').removeClass('disabled');
    }

    function clearSelectionsInOtherBlocks(thisSourceBlock, source) {
        let othersBlocks = source.not(thisSourceBlock);
        othersBlocks.find('select').val('choose-dataset');
        othersBlocks.find('input').val('');
        let uploadFile = othersBlocks.find('input[type="file"]');

        uploadFile.replaceWith(uploadFile.clone(true));
    }


    function toggleStepsBlock(step) {

        step = typeof step === 'number' ? step.toString() : step;

        if (!step) {
            step = localStorage.setItem('mpgCurrentStep', '1');
        } else {
            localStorage.setItem('mpgCurrentStep', step.toString());
        }

        $(`.wizard > div[class^="step"]:not(:has(.step-${step}))`).removeClass('active').hide();
        $(`.wizard .step-${step}`).addClass('active').show();
        $(`.wizard .step-1 .next, .wizard .step-2 .next, .wizard .step-3 .next, .wizard .step-5 .next`).addClass('disabled');

    }

    toggleStepsBlock('1');

    function renderShortcodePills(columnsList) {
        // Remove items from previous loading
        $('.mpg-dashboard .shortcode-item').remove();
        // So, we decided that user dataset has no url column, and now we got
        // available column list from server. Let's create pills from it.

        let columnDomNode = '';
        let columnArray = columnsList.split(',');

        columnArray.forEach(function (columnName) {

            columnDomNode += `<div contentEditable="false" class="shortcode-item"><span>${columnName}</span></div>`;
        });

        return columnDomNode;
    }

    function processingCreateProject(currentStep) {

        let projectNameInput = $('.wizard .step-1 input[name="project_name"]');

        const min = 111111;
        const max = 9999999;

        let randomId = Math.round(Math.random() * (max - min) + min);
        $.post(ajaxurl, {
            action: 'mpg_create_project',
            project_name: projectNameInput.val(),
            id: randomId

        }, function (response) {

            if (JSON.parse(response).success) {

                localStorage.setItem('mpg_wizard_project', JSON.stringify(
                    {projectId: randomId, projectName: projectNameInput.val()}
                    )
                );

                let neededStep = currentStep + 1;

                $('.wizard .step-2 .source form input[name="project_id"]').val(randomId);

                toggleStepsBlock(neededStep);

            } else {
                renderNotification([{type: 'danger', text: JSON.stringify(response)}]);
            }
        }).fail((error) => {
            renderNotification([{type: 'danger', text: JSON.stringify(error)}]);
        });
    }


    function processingUploadFile(context) {
        $.ajax({
            url: ajaxurl,
            type: "POST",
            data: new FormData(context),
            contentType: false,
            cache: false,
            processData: false,
            success: function (rawResponse) {

                let neededStep = parseInt(localStorage.getItem('mpgCurrentStep')) + 1;

                let response = JSON.parse(rawResponse);
console.log(response);
                if(response.validation_errors){
                    response.validation_errors.forEach(errorText => {
                        renderNotification([{type: 'warning', text: errorText}]);
                    })
                }

                let columnsList = response.columns;

                // Uploaded file has a URL column
                if (response.success && !columnsList) {

                    // jump to 2 steps ahead, because, if users file have url columns, no needed to show url builder
                    toggleStepsBlock(neededStep + 1);

                    // Uploaded file hasn't URL column
                } else if (response.success && columnsList) {

                    let columnDomNode = renderShortcodePills(columnsList);
                    // Will invoke on Wizard
                    $('.wizard .step-3 .shortcodes-pills').append(columnDomNode);

                    // Jump to 1 step to Url Builder;
                    toggleStepsBlock(neededStep);
                } else {
                    renderNotification([{'type': 'danger', 'text': 'Unknown error while processing uploaded file.'}]);
                }
            },
            complete: function(xhr, textStatus) {
                if(xhr.status === 400){

                    renderNotification([
                        {   type: 'warning',
                            text: `If uploaded file size more than 8 Mb, uploading may fall. Please, update php.ini to increase "upload_max_filesize" and "post_max_size".
                            For details, contact with your hosting provider`
                        }]);
                }
            },

            error: function (error) {
                renderNotification([{type: 'danger', text: JSON.stringify(error)}]);
            }
        });
    }

    function processingRemoteFile(remoteFileUrl) {
        $.post(
            ajaxurl,
            {
                action: 'mpg_data_source_remote_file',
                projectId: JSON.parse(localStorage.getItem('mpg_wizard_project')).projectId,
                remoteFileUrl: remoteFileUrl
            }, function (rawResponse) {

                let neededStep = parseInt(localStorage.getItem('mpgCurrentStep')) + 1;

                let response = JSON.parse(rawResponse);

                if(response.validation_errors){
                    response.validation_errors.forEach(errorText => {
                        renderNotification([{type: 'warning', text: errorText}]);
                    })
                }

                let columnsList = response.columns;

                // Uploaded file has a URL column
                if (response.success && !columnsList) {

                    // jump to 2 steps ahead, because, if users file have url columns, no needed to show url builder
                    toggleStepsBlock(neededStep + 1);

                    // Uploaded file hasn't URL column
                } else if (response.success && columnsList) {

                    let columnDomNode = renderShortcodePills(columnsList);
                    // Will invoke on Wizard
                    $('.wizard .step-3 .shortcodes-pills').append(columnDomNode);

                    // Jump to 1 step to Url Builder;
                    toggleStepsBlock(neededStep);
                } else {
                    renderNotification([{'type': 'danger', 'text': response.notifications}]);
                }
            });
    }

    function processingDatasetLibrary(selectedDatasetValue) {

        let projectId = JSON.parse(localStorage.getItem('mpg_wizard_project')).projectId;
        $.post(
            ajaxurl,
            {
                action: 'mpg_data_source_dataset',
                projectId: projectId,
                selectedDataset: selectedDatasetValue
            }, function (rawResponse) {

                let response = JSON.parse(rawResponse);
                let columnsList = response.columns;

                if (response.success && columnsList) {

                    let columnDomNode = renderShortcodePills(columnsList);
                    // Will invoke on Wizard
                    $('.wizard .step-3 .shortcodes-pills').append(columnDomNode);

                    // Jump to 1 step to Url Builder;
                    let neededStep = parseInt(localStorage.getItem('mpgCurrentStep')) + 1;
                    toggleStepsBlock(neededStep);

                } else {
                    renderNotification([{type: 'danger', text: response.notifications}]);
                }
            });
    }

    function processingCreateUrlColumn(projectId, urlTemplate, spaceReplacerValue) {

        let isNeededSitemap = $('#sitemap_settings_on').is(":checked");
        let paramsObject = {
            action: 'mpg_create_url_for_dataset',
            projectId: projectId,
            urlTemplate: urlTemplate,
            spaceReplacer: spaceReplacerValue,
            isNeededSitemap: isNeededSitemap
        };

        if (isNeededSitemap) {

            Object.assign(paramsObject, {
                sitemap_name: $('input[name="sitemap_file_name"]').val(),
                sitemap_max_url: $('input[name="sitemap_max_url"]').val(),
                sitemap_update_freq: $('select[id="sitemap_frequency_dropdown"]').val(),
                add_to_robots: $('input[name="sitemap_robot"]').is(':checked')
            });
        }

        $.post(
            ajaxurl, paramsObject, function (rawResponse) {

                let response = JSON.parse(rawResponse);

                if (response.success === true) {

                    if ($('.wizard').length) {
                        let neededStep = parseInt(localStorage.getItem('mpgCurrentStep')) + 1;
                        toggleStepsBlock(neededStep);
                    } else {
                        $('.mpg-dashboard #url-builder-modal').modal('hide');
                        $('.url-builder-input').html(''); // clear input;
                        renderNotification(response.notifications)
                        init(projectId);
                    }
                } else {
                    $('.mpg-dashboard #url-builder-modal').modal('hide');
                    $('.url-builder-input').html(''); // clear input;
                    renderNotification(response.notifications)
                }
            });
    }

    function previousButtonHandler() {

        let currentStep = parseInt(localStorage.getItem('mpgCurrentStep'));
        let neededStep = currentStep - 1;

        if (currentStep !== 1) {
            if (!$(`.wizard .step-${currentStep} .navigator  .previous`).hasClass('disabled')) {
                toggleStepsBlock(neededStep);
            }
        }
    }

    function getAvailablePages() {
        $.post(
            ajaxurl,
            {
                action: 'mpg_get_available_pages',
            }, function (rawResponse) {

                let response = JSON.parse(rawResponse);

                if (response.success === true) {

                    let pagesList = response.data;

                    let pagesNode = '';
                    for (let index in pagesList) {
                        pagesNode += `<option value="${index}">${pagesList[index]}</option>`;
                    }

                    $('.wizard .step-5 form select[name="post_template_id"]').append(pagesNode);
                }
            });
    }

    function renderUrlBuilderModal(columnList) {

        let pillsShortcodesNode = renderShortcodePills(columnList);
        // Will invoke on Dashboard modal
        $('.mpg-dashboard .modal-content .shortcodes-pills').append(pillsShortcodesNode);


        $('.mpg-dashboard #url-builder-modal').modal({
            backdrop: 'static',
            keyboard: false,
            show: true
        });

    }


    export {
        highlightBlock,
        clearSelectionsInOtherBlocks,
        toggleStepsBlock,
        processingCreateProject,
        processingUploadFile,
        processingRemoteFile,
        processingDatasetLibrary,
        processingCreateUrlColumn,
        previousButtonHandler,
        getAvailablePages,
        renderUrlBuilderModal

    };
