
import {processingCreateUrlColumn} from "./functions.js";
import {renderNotification, getDashboardUrl} from "../../helpers.js";
import {__} from "../../languages/loader.js";
import {deleteProject} from "../dashboard/create_edit_delete_project.js";

let urlTemplate;

let urlInput           = '.wizard .step-3 .url-builder-input, .mpg-dashboard .modal-content .url-builder-input';
let shortcodeItem      = '.wizard .shortcodes-pills .shortcode-item, .mpg-dashboard .shortcodes-pills .shortcode-item';
let removeShortcode    = '.wizard .url-builder-block .shortcode-item span, .mpg-dashboard .url-builder-block .shortcode-item span';

(function ($) {
    $(document).on('click', shortcodeItem, function () {

        let thisElement = $(this).get(0);
        let cloneElement = $(thisElement).clone();

        $(urlInput).append(cloneElement).trigger('input');
    });

    $(document).on('click', removeShortcode, function () {

        $(this).parents('.shortcode-item').remove();

    });

    function parseUrlBuilderInput(rawString) {
        // baseUrl - is a global var that get value in ajax request in app.js
        return `/${rawString.replace(/\s/g, "")}/`;
    }

    $(urlInput).on('input', function (e) {

        let nextButton = $('.wizard .step-3 .navigator .next, .mpg-dashboard .navigator .next');

        if ($(this).text().length) {
            nextButton.removeClass('disabled');
        } else {
            nextButton.addClass('disabled');
        }

        let rawUrlInput = $(this).text();

        urlTemplate = parseUrlBuilderInput(rawUrlInput);

        $('.result-url').text(`${baseUrl}${urlTemplate}`);
    });


    $('.wizard .step-3 .navigator .next').on('click', function () {

        if ($(urlInput).text().indexOf('mpg_') !== -1) {

            let spaceReplacerValue = $('.space-replacer-select').val();
            let projectId = JSON.parse(localStorage.getItem('mpg_wizard_project')).projectId;

            processingCreateUrlColumn(projectId, urlTemplate, spaceReplacerValue);

        } else {

            renderNotification([{
                type: 'warning',
                text: __('You need to use at least one header name in your URL')
            }]);
        }
    });

    $('.mpg-dashboard .navigator .next').on('click', function () {

        if ($(urlInput).text().indexOf('mpg_') !== -1) {
            let spaceReplacerValue = $('.space-replacer-select').val();
            let projectId = localStorage.getItem('mpg_current_tab_id');

            processingCreateUrlColumn(projectId, urlTemplate, spaceReplacerValue);

        } else {
            renderNotification([{
                type: 'warning',
                text: __('You need to use at least one header name in your URL')
            }]);
        }
    });

    $('.navigator .cancel').on('click', function () {
        // If it's Wizard - need to remove project.
        if ($('.wizard .step-1.active').length) {

            location.href = getDashboardUrl();
            // At least one of them (not a first step)
        } else if ($('.wizard .step-2.active, .wizard .step-3.active, .wizard .step-4.active, .wizard .step-5.active').length) {
            // If user canceled creation of project. We need to delete not fully created project.
            let projectId = JSON.parse(localStorage.getItem('mpg_wizard_project')).projectId;

            deleteProject(projectId);
            localStorage.removeItem('mpg_wizard_project');
            location.href = getDashboardUrl();

        } else {

            localStorage.removeItem('mpg_wizard_project');
            // If it's modal window - just hide modal
            // todo maybe needed to delete project too ?
            $('.mpg-dashboard #url-builder-modal').modal('hide');

            renderNotification([{
                type: 'warning',
                text: __('Creation of URL was cancelled. Use file that already has "url" column, or create URL in UrlBuilder.')
            }])
        }

    })

}(jQuery));