<?php

/**
 * Plugin Name: Multiple Pages Generator by Porthas
 * Plugin URI: https://www.porthas.com/
 * Description: Plugin for generation of multiple frontend pages from CSV data file.
 * Author: <a href='https://www.porthas.com/'>Porthas inc</a>
 * Author URI: https://www.porthas.com/
 * Version: 1.5.2
 */
// Define Freemius API
if ( !defined( 'ABSPATH' ) ) {
    exit;
}

if ( !function_exists( 'mpg_app' ) ) {
    // Create a helper function for easy SDK access.
    function mpg_app()
    {
        global  $mpg_app ;
        
        if ( !isset( $mpg_app ) ) {
            // Activate multisite network integration.
            if ( !defined( 'WP_FS__PRODUCT_2867_MULTISITE' ) ) {
                define( 'WP_FS__PRODUCT_2867_MULTISITE', true );
            }
            // Include Freemius SDK.
            require_once dirname( __FILE__ ) . '/freemius/start.php';
            $mpg_app = fs_dynamic_init( array(
                'id'             => '2867',
                'slug'           => 'multi-pages-plugin',
                'type'           => 'plugin',
                'public_key'     => 'pk_0c4166c5c398b9b750cf04df59173',
                'is_premium'     => false,
                'premium_suffix' => 'Pro',
                'has_addons'     => false,
                'has_paid_plans' => true,
                'menu'           => array(
                'slug'       => 'multi-pages-plugin',
                'first-path' => 'admin.php?page=multi-pages-plugin',
                'support'    => false,
            ),
                'is_live'        => true,
            ) );
        }
        
        return $mpg_app;
    }
    
    // Init Freemius.
    mpg_app();
    // Signal that SDK was initiated.
    do_action( 'mpg_app_loaded' );
}

add_action( 'plugins_loaded', 'true_load_plugin_textdomain' );
function true_load_plugin_textdomain()
{
    load_plugin_textdomain( 'mpg', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
}

add_action(
    'schedule_execution',
    'scheduled_cron_handler',
    10,
    5
);
require_once plugin_dir_path( __FILE__ ) . 'constants.php';
require_once MULTIPAGE_PLUGIN_PATH . 'includes/datasets/datasets.php';
require_once MULTIPAGE_PLUGIN_PATH . 'includes/options.php';
require_once MULTIPAGE_PLUGIN_PATH . 'includes/functions.php';
// Register additional intervals for cron
add_filter( 'cron_schedules', 'cron_weekly' );
add_filter( 'cron_schedules', 'cron_monthly' );
// Redirect user to "Welcome screen" after plugin activation.
register_activation_hook( __FILE__, 'mpg_set_activation_option' );
add_action( 'admin_init', 'mpg_activation_tasks' );
// Remove cron tasks (if any) after plugin deactivated;
register_deactivation_hook( __FILE__, 'mpg_set_deactivation_option' );
/**
 * @param array $atts
 * @param string $content (html content)
 * @return bool|string
 */
add_shortcode( 'mpg', 'mpgGroupAction' );
function mpgGroupAction( $atts = array(), $content = null )
{
    if ( is_admin() ) {
        return false;
    }
    $atts = array_change_key_case( (array) $atts, CASE_LOWER );
    // normalize attribute keys, lowercase
    $whereParams = explode( ',', $atts['where'] );
    $groupByColumnName = ( isset( $atts['group'] ) ? $atts['group'] : null );
    if ( empty($whereParams) ) {
        return 'Cannot find "where" params';
    }
    $template = preg_replace( "/{{/", "{{mpg_", $content );
    $urls_and_project_id = get_urls_and_project_id();
    $project_id = $urls_and_project_id['project_id'];
    $data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/' . getSiteName() . '-' . $project_id . '-settings.json';
    $default_data_file_path = MULTIPAGE_PLUGIN_PATH . '/uploaded_templates/default-settings.json';
    $data_file_content = ( file_exists( $data_file_path ) ? file_get_contents( $data_file_path ) : file_get_contents( $default_data_file_path ) );
    $data_file_content_array = json_decode( $data_file_content, true );
    $project_object = get_option( 'multipages_project_' . $project_id );
    array_shift( $data_file_content_array );
    $settingsTable = array_merge( [ json_decode( $project_object['csv_settings_shortcodes'], true ) ], $data_file_content_array );
    
    if ( $settingsTable ) {
        $columnNames = array_shift( $settingsTable );
        $groupByColumnKey = array_search( $groupByColumnName, $columnNames );
        $groupSettings = [];
        foreach ( $whereParams as $gParam ) {
            $gParam = trim( $gParam );
            if ( empty($gParam) ) {
                continue;
            }
            $paramValues = explode( '=', $gParam );
            $colName = strtolower( trim( $paramValues[0] ) );
            // column name
            $keyColumn = array_search( $colName, $columnNames );
            $groupSettings[$keyColumn] = trim( $paramValues[1] );
            // value for search
        }
        foreach ( $settingsTable as $string ) {
            foreach ( $groupSettings as $column => $value ) {
                if ( strtolower( $value ) != strtolower( $string[$column] ) ) {
                    continue 2;
                }
            }
            
            if ( !empty($groupByColumnKey) ) {
                $dataSet[strtolower( $string[$groupByColumnKey] )] = $string;
            } else {
                $dataSet[] = $string;
            }
        
        }
        if ( empty($dataSet) ) {
            return false;
        }
        return mpgScReplacer(
            $template,
            $dataSet,
            $urls_and_project_id['url'],
            $urls_and_project_id['project_id']
        );
    }
    
    return false;
}

function get_urls_and_project_id()
{
    $redirect_rules = get_redirect_rules();
    // If requested URL is available in array of all URL's
    $index_of_element = array_search( getRequestUri(), array_column( $redirect_rules, 'url' ) );
    return [
        'url'         => array_column( $redirect_rules, 'url' ),
        'project_id'  => ( $redirect_rules[$index_of_element] ? $redirect_rules[$index_of_element]['project_id'] : null ),
        'template_id' => ( $redirect_rules[$index_of_element] ? $redirect_rules[$index_of_element]['template_id'] : null ),
    ];
}

/**
 * Create the virtual page with content from template and set settings for view this page like normal WP page
 * @return void
 */
add_action( 'template_redirect', 'mpgViewMultiPages' );
function mpgViewMultiPages()
{
    global  $wp_query ;
    $redirect_rules = get_redirect_rules();
    
    if ( $redirect_rules ) {
        // If requested URL is available in array of all URL's
        $index_of_element = array_search( getRequestUri(), array_column( $redirect_rules, 'url' ) );
        
        if ( $index_of_element !== false ) {
            $template_post_id = $redirect_rules[$index_of_element]['template_id'];
            $post = get_post( $template_post_id );
            // if post not exist in WP database then create multi page from plugin
            
            if ( is_404() ) {
                // do changes in title and content
                $urls_array = array_column( $redirect_rules, 'url' );
                $project_id = $redirect_rules[$index_of_element]['project_id'];
                $post->post_title = mpgScReplacer(
                    $post->post_title,
                    [],
                    $urls_array,
                    $project_id
                );
                $post->post_content = mpgScReplacer(
                    $post->post_content,
                    [],
                    $urls_array,
                    $project_id
                );
                // setup template post as global, this is needed for the_title(), the_permalink()
                setup_postdata( $GLOBALS['post'] =& $post );
                // set the post as cached, it is necessary for the get_post () function, it will return the replaced data when this function is called
                wp_cache_set( $post->ID, $post, 'posts' );
                // set status code 200, because on default this page not exist and return 404 code
                status_header( 200 );
                // set important settings for page query
                $wp_query->queried_object = $post;
                $wp_query->is_single = false;
                $wp_query->is_404 = false;
                $wp_query->queried_object_id = $post->ID;
                $wp_query->post_count = 1;
                $wp_query->current_post = -1;
                $wp_query->posts = array( $post );
                $wp_query->is_author = false;
                $wp_query->is_page = true;
                $wp_query->is_singular = true;
            } else {
                // view message if post exist in WP database
                echo  '<script>console.warn(' . __( 'Multiple Pages Generator >> This URL is already created in WordPress. Can not replace content.', 'mpg' ) . ');</script>' ;
            }
        
        }
    
    }

}

/**
 * Start load object of page content
 */
add_action( 'get_header', 'multipage_start_ob', 1 );
function multipage_start_ob()
{
    $currentPost = get_post();
    $all_post_templates_ids = getAllTepmlatesIds();
    if ( in_array( $currentPost->ID, $all_post_templates_ids ) ) {
        ob_start( 'multipage_replace_data' );
    }
}

/**
 * End load object of page content
 */
add_action( 'get_footer', 'multipage_end_ob', 1 );
function multipage_end_ob()
{
    $currentPost = get_post();
    $all_post_templates_ids = getAllTepmlatesIds();
    if ( in_array( $currentPost->ID, $all_post_templates_ids ) ) {
        ob_end_flush();
    }
}
